@@ -1,9 +1,7 @@
-/*	$calcurse: apoint.c,v 1.38 2010-03-20 10:54:41 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -43,7 +41,7 @@
 
 #include "calcurse.h"
 
-struct apoint_list   *alist_p;
+llist_ts_t            alist_p;
 static struct apoint  bkp_cut_apoint;
 static int            hilt;
 
@@ -58,6 +56,14 @@ apoint_free_bkp (enum eraseflg flag)
   erase_note (&bkp_cut_apoint.note, flag);
 }
 
+static void
+apoint_free (struct apoint *apt)
+{
+  mem_free (apt->mesg);
+  erase_note (&apt->note, ERASE_FORCE_KEEP_NOTE);
+  mem_free (apt);
+}
+
 static void
 apoint_dup (struct apoint *in, struct apoint *bkp)
 {
@@ -74,9 +80,7 @@ apoint_dup (struct apoint *in, struct apoint *bkp)
 void
 apoint_llist_init (void)
 {
-  alist_p = mem_malloc (sizeof (struct apoint_list));
-  alist_p->root = NULL;
-  pthread_mutex_init (&(alist_p->mutex), NULL);
+  LLIST_TS_INIT (&alist_p);
 }
 
 /*
@@ -87,18 +91,8 @@ apoint_llist_init (void)
 void
 apoint_llist_free (void)
 {
-  struct apoint *o, **i;
-
-  i = &alist_p->root;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o->mesg);
-      erase_note (&o->note, ERASE_FORCE_KEEP_NOTE);
-      mem_free (o);
-    }
-  mem_free (alist_p);
+  LLIST_TS_FREE_INNER (&alist_p, apoint_free);
+  LLIST_TS_FREE (&alist_p);
 }
 
 /* Sets which appointment is highlighted. */
@@ -127,36 +121,32 @@ apoint_hilt (void)
   return (hilt);
 }
 
+static int
+apoint_cmp_start (struct apoint *a, struct apoint *b)
+{
+  return (a->start < b->start ? -1 : (a->start == b->start ? 0 : 1));
+}
+
 struct apoint *
 apoint_new (char *mesg, char *note, long start, long dur, char state)
 {
-  struct apoint *o, **i;
+  struct apoint *apt;
 
-  o = mem_malloc (sizeof (struct apoint));
-  o->mesg = mem_strdup (mesg);
-  o->note = (note != NULL) ? mem_strdup (note) : NULL;
-  o->state = state;
-  o->start = start;
-  o->dur = dur;
+  apt = mem_malloc (sizeof (struct apoint));
+  apt->mesg = mem_strdup (mesg);
+  apt->note = (note != NULL) ? mem_strdup (note) : NULL;
+  apt->state = state;
+  apt->start = start;
+  apt->dur = dur;
 
-  pthread_mutex_lock (&(alist_p->mutex));
-  i = &alist_p->root;
-  for (;;)
-    {
-      if (*i == 0 || (*i)->start > start)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
-    }
-  pthread_mutex_unlock (&(alist_p->mutex));
+  LLIST_TS_LOCK (&alist_p);
+  LLIST_TS_ADD_SORTED (&alist_p, apt, apoint_cmp_start);
+  LLIST_TS_UNLOCK (&alist_p);
 
-  return (o);
+  return apt;
 }
 
-/* 
+/*
  * Add an item in either the appointment or the event list,
  * depending if the start time is entered or not.
  */
@@ -184,65 +174,68 @@ apoint_add (void)
   int is_appointment = 1;
 
   /* Get the starting time */
-  while (check_time (item_time) != 1)
+  do
     {
       status_mesg (mesg_1, "");
       if (getstring (win[STA].p, item_time, LTIME, 0, 1) != GETSTRING_ESC)
-	{
-	  if (strlen (item_time) == 0)
-	    {
-	      is_appointment = 0;
-	      break;
-	    }
-	  else if (check_time (item_time) != 1)
-	    {
-	      status_mesg (format_message_1, enter_str);
-	      (void)wgetch (win[STA].p);
-	    }
-	  else
-	    (void)sscanf (item_time, "%u:%u", &heures, &minutes);
-	}
+        {
+          if (strlen (item_time) == 0)
+            {
+              is_appointment = 0;
+              break;
+            }
+          else if (check_time (item_time) != 1)
+            {
+              status_mesg (format_message_1, enter_str);
+              (void)wgetch (win[STA].p);
+            }
+          else
+            (void)sscanf (item_time, "%u:%u", &heures, &minutes);
+        }
       else
-	return;
+        return;
     }
-  /* 
-   * Check if an event or appointment is entered, 
-   * depending on the starting time, and record the 
+  while (check_time (item_time) != 1);
+
+  /*
+   * Check if an event or appointment is entered,
+   * depending on the starting time, and record the
    * corresponding item.
    */
   if (is_appointment)
     {				/* Get the appointment duration */
       item_time[0] = '\0';
-      while (check_time (item_time) == 0)
-	{
-	  status_mesg (mesg_2, "");
-	  if (getstring (win[STA].p, item_time, LTIME, 0, 1) != GETSTRING_VALID)
-	    return;		//nothing entered, cancel adding of event
-	  else if (check_time (item_time) == 0)
-	    {
-	      status_mesg (format_message_2, enter_str);
-	      (void)wgetch (win[STA].p);
-	    }
-	  else
-	    {
-	      if (check_time (item_time) == 2)
-		apoint_duration = atoi (item_time);
-	      else if (check_time (item_time) == 1)
-		{
-		  (void)sscanf (item_time, "%u:%u", &end_h, &end_m);
-		  if (end_h < heures)
-		    {
-		      apoint_duration = MININSEC - minutes + end_m
+      do
+        {
+          status_mesg (mesg_2, "");
+          if (getstring (win[STA].p, item_time, LTIME, 0, 1) != GETSTRING_VALID)
+            return;		//nothing entered, cancel adding of event
+          else if (check_time (item_time) == 0)
+            {
+              status_mesg (format_message_2, enter_str);
+              (void)wgetch (win[STA].p);
+            }
+          else
+            {
+              if (check_time (item_time) == 2)
+                apoint_duration = atoi (item_time);
+              else if (check_time (item_time) == 1)
+                {
+                  (void)sscanf (item_time, "%u:%u", &end_h, &end_m);
+                  if (end_h < heures || ((end_h == heures) && (end_m < minutes)))
+                    {
+                      apoint_duration = MININSEC - minutes + end_m
                         + (24 + end_h - (heures + 1)) * MININSEC;
-		    }
-		  else
-		    {
-		      apoint_duration = MININSEC - minutes
+                    }
+                  else
+                    {
+                      apoint_duration = MININSEC - minutes
                         + end_m + (end_h - (heures + 1)) * MININSEC;
-		    }
-		}
-	    }
-	}
+                    }
+                }
+            }
+        }
+      while (check_time (item_time) == 0);
     }
   else				/* Insert the event Id */
     Id = 1;
@@ -251,19 +244,19 @@ apoint_add (void)
   if (getstring (win[STA].p, item_mesg, BUFSIZ, 0, 1) == GETSTRING_VALID)
     {
       if (is_appointment)
-	{
-	  apoint_start = date2sec (*calendar_get_slctd_day (), heures, minutes);
-	  (void)apoint_new (item_mesg, 0L, apoint_start,
+        {
+          apoint_start = date2sec (*calendar_get_slctd_day (), heures, minutes);
+          (void)apoint_new (item_mesg, 0L, apoint_start,
                             min2sec (apoint_duration), 0L);
-	  if (notify_bar ())
-	    notify_check_added (item_mesg, apoint_start, 0L);
-	}
+          if (notify_bar ())
+            notify_check_added (item_mesg, apoint_start, 0L);
+        }
       else
-	(void)event_new (item_mesg, 0L,
+        (void)event_new (item_mesg, 0L,
                          date2sec (*calendar_get_slctd_day (), 12, 0), Id);
 
       if (hilt == 0)
-	hilt++;
+        hilt++;
     }
   wins_erase_status_bar ();
 }
@@ -288,12 +281,12 @@ apoint_delete (struct conf *conf, unsigned *nb_events, unsigned *nb_apoints)
       status_mesg (del_app_str, choices);
       answer = wgetch (win[STA].p);
       if ((answer == 'y') && (nb_items != 0))
-	go_for_deletion = 1;
+        go_for_deletion = 1;
       else
-	{
-	  wins_erase_status_bar ();
-	  return;
-	}
+        {
+          wins_erase_status_bar ();
+          return;
+        }
     }
   else if (nb_items != 0)
     go_for_deletion = 1;
@@ -301,31 +294,31 @@ apoint_delete (struct conf *conf, unsigned *nb_events, unsigned *nb_apoints)
   if (go_for_deletion)
     {
       if (nb_items != 0)
-	{
-	  deleted_item_type = day_erase_item (date, hilt, ERASE_DONT_FORCE);
-	  if (deleted_item_type == EVNT || deleted_item_type == RECUR_EVNT)
-	    {
-	      (*nb_events)--;
-	      to_be_removed = 1;
-	    }
-	  else if (deleted_item_type == APPT || deleted_item_type == RECUR_APPT)
-	    {
-	      (*nb_apoints)--;
-	      to_be_removed = 3;
-	    }
-	  else if (deleted_item_type == 0)
+        {
+          deleted_item_type = day_erase_item (date, hilt, ERASE_DONT_FORCE);
+          if (deleted_item_type == EVNT || deleted_item_type == RECUR_EVNT)
+            {
+              (*nb_events)--;
+              to_be_removed = 1;
+            }
+          else if (deleted_item_type == APPT || deleted_item_type == RECUR_APPT)
+            {
+              (*nb_apoints)--;
+              to_be_removed = 3;
+            }
+          else if (deleted_item_type == 0)
             return;
-	  else
+          else
             EXIT (_("no such type"));
-	  /* NOTREACHED */
+          /* NOTREACHED */
 
-	  if (hilt > 1)
-	    hilt--;
-	  if (apad.first_onscreen >= to_be_removed)
-	    apad.first_onscreen = apad.first_onscreen - to_be_removed;
-	  if (nb_items == 1)
-	    hilt = 0;
-	}
+          if (hilt > 1)
+            hilt--;
+          if (apad.first_onscreen >= to_be_removed)
+            apad.first_onscreen = apad.first_onscreen - to_be_removed;
+          if (nb_items == 1)
+            hilt = 0;
+        }
     }
 }
 
@@ -334,13 +327,12 @@ int
 apoint_cut (unsigned *nb_events, unsigned *nb_apoints)
 {
   const int NBITEMS = *nb_apoints + *nb_events;
-  int item_type, to_be_removed;  
+  int item_type, to_be_removed;
   long date;
 
   if (NBITEMS == 0)
     return 0;
 
-  to_be_removed = 0;
   date = calendar_get_slctd_day_sec ();
   item_type = day_cut_item (date, hilt);
   if (item_type == EVNT || item_type == RECUR_EVNT)
@@ -356,7 +348,7 @@ apoint_cut (unsigned *nb_events, unsigned *nb_apoints)
   else
     EXIT (_("no such type"));
   /* NOTREACHED */
-  
+
   if (hilt > 1)
     hilt--;
   if (apad.first_onscreen >= to_be_removed)
@@ -382,7 +374,7 @@ apoint_paste (unsigned *nb_events, unsigned *nb_apoints, int cut_item_type)
     (*nb_apoints)++;
   else
     return;
-  
+
   if (hilt == 0)
     hilt++;
 }
@@ -483,79 +475,57 @@ apoint_scan (FILE *f, struct tm start, struct tm end, char state, char *note)
 struct apoint *
 apoint_get (long day, int pos)
 {
-  struct apoint *o;
-  int n;
+  llist_item_t *i = LLIST_TS_FIND_NTH (&alist_p, pos, day, apoint_inday);
+
+  if (i)
+    return LLIST_TS_GET_DATA (i);
 
-  n = 0;
-  for (o = alist_p->root; o; o = o->next)
-    {
-      if (apoint_inday (o, day))
-	{
-	  if (n == pos)
-	    return (o);
-	  n++;
-	}
-    }
   EXIT (_("item not found"));
-  return 0;
-  /* NOTREACHED */  
+  /* NOTREACHED */
 }
 
 void
 apoint_delete_bynum (long start, unsigned num, enum eraseflg flag)
 {
-  unsigned n;
+  llist_item_t *i;
   int need_check_notify = 0;
-  struct apoint *i, **iptr;
 
-  n = 0;
-  pthread_mutex_lock (&(alist_p->mutex));
-  iptr = &alist_p->root;
-  for (i = alist_p->root; i != 0; i = i->next)
+  LLIST_TS_LOCK (&alist_p);
+  i = LLIST_TS_FIND_NTH (&alist_p, num, start, apoint_inday);
+
+  if (!i)
+    EXIT (_("no such appointment"));
+  struct apoint *apt = LLIST_TS_GET_DATA (i);
+
+  switch (flag)
     {
-      if (apoint_inday (i, start))
-	{
-	  if (n == num)
-	    {
-              switch (flag)
-                {
-                case ERASE_FORCE_ONLY_NOTE:
-                  erase_note (&i->note, flag);
-                  pthread_mutex_unlock (&(alist_p->mutex));
-                  break;
-                case ERASE_CUT:
-                  apoint_free_bkp (ERASE_FORCE);
-                  apoint_dup (i, &bkp_cut_apoint);
-                  erase_note (&i->note, ERASE_FORCE_KEEP_NOTE);
-                  /* FALLTHROUGH */
-                default:
-		  if (notify_bar ())
-		    need_check_notify = notify_same_item (i->start);
-		  *iptr = i->next;
-		  mem_free (i->mesg);
-                  if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
-                    erase_note (&i->note, flag);
-		  mem_free (i);
-		  pthread_mutex_unlock (&(alist_p->mutex));
-		  if (need_check_notify)
-		    notify_check_next_app ();
-                  break;
-		}
-	      return;
-	    }
-	  n++;
-	}
-      iptr = &i->next;
+    case ERASE_FORCE_ONLY_NOTE:
+      erase_note (&apt->note, flag);
+      break;
+    case ERASE_CUT:
+      apoint_free_bkp (ERASE_FORCE);
+      apoint_dup (apt, &bkp_cut_apoint);
+      erase_note (&apt->note, ERASE_FORCE_KEEP_NOTE);
+      /* FALLTHROUGH */
+    default:
+      if (notify_bar ())
+        need_check_notify = notify_same_item (apt->start);
+      LLIST_TS_REMOVE (&alist_p, i);
+      mem_free (apt->mesg);
+      if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
+        erase_note (&apt->note, flag);
+      mem_free (apt);
+      if (need_check_notify)
+        notify_check_next_app (0);
+      break;
     }
-  
-  pthread_mutex_unlock (&(alist_p->mutex));
-  EXIT (_("no such appointment"));
-  /* NOTREACHED */
+
+  LLIST_TS_UNLOCK (&alist_p);
 }
 
 /*
  * Return the line number of an item (either an appointment or an event) in
- * the appointment panel. This is to help the appointment scroll function 
+ * the appointment panel. This is to help the appointment scroll function
  * to place beggining of the pad correctly.
  */
 static int
@@ -572,9 +542,9 @@ get_item_line (int item_nb, int nb_events_inday)
   return line;
 }
 
-/* 
+/*
  * Update (if necessary) the first displayed pad line to make the
- * appointment panel scroll down next time pnoutrefresh is called. 
+ * appointment panel scroll down next time pnoutrefresh is called.
  */
 void
 apoint_scroll_pad_down (int nb_events_inday, int win_length)
@@ -594,9 +564,9 @@ apoint_scroll_pad_down (int nb_events_inday, int win_length)
     apad.first_onscreen = item_last_line - awin_length;
 }
 
-/* 
+/*
  * Update (if necessary) the first displayed pad line to make the
- * appointment panel scroll up next time pnoutrefresh is called. 
+ * appointment panel scroll up next time pnoutrefresh is called.
  */
 void
 apoint_scroll_pad_up (int nb_events_inday)
@@ -608,6 +578,12 @@ apoint_scroll_pad_up (int nb_events_inday)
     apad.first_onscreen = item_first_line;
 }
 
+static int
+apoint_starts_after (struct apoint *apt, long time)
+{
+  return (apt->start > time);
+}
+
 /*
  * Look in the appointment list if we have an item which starts before the item
  * stored in the notify_app structure (which is the next item to be notified).
@@ -615,35 +591,32 @@ apoint_scroll_pad_up (int nb_events_inday)
 struct notify_app *
 apoint_check_next (struct notify_app *app, long start)
 {
-  struct apoint *i;
+  llist_item_t *i;
 
-  pthread_mutex_lock (&(alist_p->mutex));
-  for (i = alist_p->root; i != 0; i = i->next)
+  LLIST_TS_LOCK (&alist_p);
+  i = LLIST_TS_FIND_FIRST (&alist_p, start, apoint_starts_after);
+
+  if (i)
     {
-      if (i->start > app->time)
-	{
-	  pthread_mutex_unlock (&(alist_p->mutex));
-	  return (app);
-	}
-      else
-	{
-	  if (i->start > start)
-	    {
-	      app->time = i->start;
-	      app->txt = mem_strdup (i->mesg);
-	      app->state = i->state;
-	      app->got_app = 1;
-	    }
-	}
+      struct apoint *apt = LLIST_TS_GET_DATA (i);
+
+      if (apt->start <= app->time)
+        {
+          app->time = apt->start;
+          app->txt = mem_strdup (apt->mesg);
+          app->state = apt->state;
+          app->got_app = 1;
+        }
     }
-  pthread_mutex_unlock (&(alist_p->mutex));
+
+  LLIST_TS_UNLOCK (&alist_p);
 
   return (app);
 }
 
-/* 
- * Returns a structure of type struct apoint_list given a structure of type 
- * recur_apoint_s 
+/*
+ * Returns a structure of type struct apoint_list given a structure of type
+ * recur_apoint_s
  */
 struct apoint *
 apoint_recur_s2apoint_s (struct recur_apoint *p)
@@ -663,10 +636,9 @@ apoint_recur_s2apoint_s (struct recur_apoint *p)
 void
 apoint_switch_notify (void)
 {
-  struct apoint *apoint;
   struct day_item *p;
   long date;
-  int apoint_nb = 0, n, need_chk_notify;
+  int apoint_nb = 0, need_chk_notify;
 
   p = day_get_item (hilt);
   if (p->type != APPT && p->type != RECUR_APPT)
@@ -682,34 +654,18 @@ apoint_switch_notify (void)
   else if (p->type == APPT)
     apoint_nb = day_item_nb (date, hilt, APPT);
 
-  n = 0;
   need_chk_notify = 0;
-  pthread_mutex_lock (&(alist_p->mutex));
+  LLIST_TS_LOCK (&alist_p);
 
-  for (apoint = alist_p->root; apoint != 0; apoint = apoint->next)
-    {
-      if (apoint_inday (apoint, date))
-	{
-	  if (n == apoint_nb)
-	    {
-	      apoint->state ^= APOINT_NOTIFY;
-	      if (notify_bar ())
-                {
-                  notify_check_added (apoint->mesg, apoint->start,
-                                      apoint->state);
-                }
-	      pthread_mutex_unlock (&(alist_p->mutex));
-	      if (need_chk_notify)
-		notify_check_next_app ();
-	      return;
-	    }
-	  n++;
-	}
-    }
+  struct apoint *apt = apoint_get (date, apoint_nb);
 
-  pthread_mutex_unlock (&(alist_p->mutex));  
-  EXIT (_("no such appointment"));
-  /* NOTREACHED */  
+  apt->state ^= APOINT_NOTIFY;
+  if (notify_bar ())
+    notify_check_added (apt->mesg, apt->start, apt->state);
+  if (need_chk_notify)
+    notify_check_next_app (0);
+
+  LLIST_TS_UNLOCK (&alist_p);
 }
 
 /* Updates the Appointment panel */
@@ -737,8 +693,8 @@ apoint_update_panel (int which_pan)
                      win[APP].h - 2);
   custom_apply_attr (win[APP].p, ATTR_HIGHEST);
   mvwprintw (win[APP].p, title_lines, title_xpos, "%s  %s %d, %d",
-	     calendar_get_pom (date), _(monthnames[slctd_date.mm - 1]),
-	     slctd_date.dd, slctd_date.yyyy);
+             calendar_get_pom (date), _(monthnames[slctd_date.mm - 1]),
+             slctd_date.dd, slctd_date.yyyy);
   custom_remove_attr (win[APP].p, ATTR_HIGHEST);
 
   /* Draw the scrollbar if necessary. */
@@ -751,16 +707,16 @@ apoint_update_panel (int which_pan)
       int sbar_top = highend + title_lines + 1;
 
       if ((sbar_top + sbar_length) > win[APP].h - 1)
-	sbar_length = win[APP].h - 1 - sbar_top;
+        sbar_length = win[APP].h - 1 - sbar_top;
       draw_scrollbar (win[APP].p, sbar_top, win[APP].w - 2, sbar_length,
-		      title_lines + 1, win[APP].h - 1, hilt_bar);
+                      title_lines + 1, win[APP].h - 1, hilt_bar);
     }
 
   wnoutrefresh (win[APP].p);
   pnoutrefresh (apad.ptrwin, apad.first_onscreen, 0,
-		win[APP].y + title_lines + 1, win[APP].x + bordr,
-		win[APP].y + win[APP].h - 2 * bordr,
-		win[APP].x + win[APP].w - 3 * bordr);
+                win[APP].y + title_lines + 1, win[APP].x + bordr,
+                win[APP].y + win[APP].h - 2 * bordr,
+                win[APP].x + win[APP].w - 3 * bordr);
 }
 
 void
@@ -773,5 +729,9 @@ apoint_paste_item (void)
   (void)apoint_new (bkp_cut_apoint.mesg, bkp_cut_apoint.note,
                     bkp_start, bkp_cut_apoint.dur,
                     bkp_cut_apoint.state);
+
+  if (notify_bar ())
+    notify_check_added (bkp_cut_apoint.mesg, bkp_start, bkp_cut_apoint.state);
+
   apoint_free_bkp (ERASE_FORCE_KEEP_NOTE);
 }
