@@ -1,9 +1,7 @@
-/*	$Id: calcurse.h,v 1.5 2010-03-22 08:16:33 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -56,7 +54,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include "llist.h"
 #include "htable.h"
+#include "llist_ts.h"
 
 /* Internationalization. */
 #if ENABLE_NLS
@@ -140,7 +140,7 @@
 #define TAB       		9
 #define SPACE                   32
 
-#define KEYS_KEYLEN		3 /* length of each keybinding */  
+#define KEYS_KEYLEN		3 /* length of each keybinding */
 #define KEYS_LABELEN		8 /* length of command description */
 #define KEYS_CMDS_PER_LINE	6 /* max number of commands per line */
 
@@ -256,7 +256,6 @@ struct date {
 /* Appointment definition. */
 struct apoint
 {
-  struct apoint *next;    /* appointments are stored in a linked-list */
   long           start;   /* seconds since 1 jan 1970 */
   long           dur;     /* duration of the appointment in seconds */
 
@@ -264,21 +263,13 @@ struct apoint
 #define APOINT_NOTIFY    0x1	/* Item needs to be notified */
 #define APOINT_NOTIFIED  0x2	/* Item was already notified */
   int            state;
-  
+
   char          *mesg;
   char          *note;
 };
 
-/* Appointments are stored in a linked-list. */
-struct apoint_list
-{
-  struct apoint   *root;
-  pthread_mutex_t  mutex;
-};
-
 /* Event definition. */
 struct event {
-  struct event *next;
   int           id;    /* event identifier */
   long          day;   /* seconds since 1 jan 1970 */
   char         *mesg;
@@ -287,7 +278,6 @@ struct event {
 
 /* Todo item definition. */
 struct todo {
-  struct todo  *next;
   char         *mesg;
   int           id;
   char         *note;
@@ -301,7 +291,6 @@ struct day_items_nb {
 
 /* Generic item description (to hold appointments, events...). */
 struct day_item {
-  struct day_item *next;
   long             start;    /* seconds since 1 jan 1970 */
   long             appt_dur; /* appointment duration in seconds */
   int              type;     /* (recursive or normal) event or appointment */
@@ -312,8 +301,7 @@ struct day_item {
   char            *note;     /* note attached to item */
 };
 
-struct days {
-  struct days *next;
+struct excp {
   long         st;    /* beggining of the considered day, in seconds */
 };
 
@@ -323,7 +311,7 @@ enum recur_type {
   RECUR_WEEKLY,
   RECUR_MONTHLY,
   RECUR_YEARLY,
-  RECUR_TYPES  
+  RECUR_TYPES
 };
 
 /* To describe an item's repetition. */
@@ -335,9 +323,8 @@ struct rpt {
 
 /* Recurrent appointment definition. */
 struct recur_apoint {
-  struct recur_apoint  *next;
   struct rpt           *rpt;   /* information about repetition */
-  struct days          *exc;   /* days when the item should not be repeated */
+  llist_t               exc;   /* days when the item should not be repeated */
   long                  start; /* beggining of the appointment */
   long                  dur;   /* duration of the appointment */
   char                  state; /* 8 bits to store item state */
@@ -345,17 +332,10 @@ struct recur_apoint {
   char                 *note;  /* note attached to appointment */
 };
 
-/* Recurrent appointments are stored in a linked-list. */
-struct recur_apoint_list {
-  struct recur_apoint  *root;
-  pthread_mutex_t       mutex;
-};
-
 /* Reccurent event definition. */
 struct recur_event {
-  struct recur_event  *next;
   struct rpt          *rpt;  /* information about repetition */
-  struct days         *exc;  /* days when the item should not be repeated */
+  llist_t              exc;  /* days when the item should not be repeated */
   int                  id;   /* event type */
   long                 day;  /* day at which event occurs */
   char                *mesg; /* event description */
@@ -369,7 +349,7 @@ struct notify_app {
   char             state;
   pthread_mutex_t  mutex;
 };
-  
+
 struct io_file {
   FILE *fd;
   char  name[BUFSIZ];
@@ -401,7 +381,7 @@ enum key {
   KEY_GENERIC_SCROLL_DOWN,
   KEY_GENERIC_SCROLL_UP,
   KEY_GENERIC_GOTO_TODAY,
-  
+
   KEY_MOVE_RIGHT,
   KEY_MOVE_LEFT,
   KEY_MOVE_DOWN,
@@ -418,7 +398,7 @@ enum key {
   KEY_VIEW_NOTE,
   KEY_RAISE_PRIORITY,
   KEY_LOWER_PRIORITY,
-  
+
   NBKEYS,
   KEY_UNDEF
 };
@@ -497,9 +477,9 @@ enum eraseflg {
   ERASE_FORCE,
   ERASE_FORCE_KEEP_NOTE,
   ERASE_FORCE_ONLY_NOTE,
-  ERASE_CUT  
+  ERASE_CUT
 };
-  
+
 /* Return codes for the getstring() function. */
 enum getstr {
   GETSTRING_VALID,
@@ -564,7 +544,7 @@ enum save_display {
 };
 
 /* apoint.c */
-extern struct apoint_list  *alist_p;
+extern llist_ts_t alist_p;
 void               apoint_free_bkp (enum eraseflg);
 void               apoint_llist_init (void);
 void               apoint_llist_free (void);
@@ -654,8 +634,9 @@ void   dmon_start (int);
 void   dmon_stop (void);
 
 /* event.c */
-extern struct event *eventlist;
+extern llist_t eventlist;
 void          event_free_bkp (enum eraseflg);
+void          event_llist_init (void);
 void          event_llist_free (void);
 struct event *event_new (char *, char *, long, int);
 unsigned      event_inday (struct event *, long);
@@ -698,6 +679,7 @@ void             io_stop_psave_thread (void);
 void             io_set_lock (void);
 unsigned         io_dump_pid (char *);
 unsigned         io_get_pid (char *);
+int              io_file_is_empty (char *);
 
 /* keys.c */
 void      keys_init (void);
@@ -749,7 +731,7 @@ void   mem_stats (void);
 #  define mem_realloc(p, n, s)  xrealloc ((p), (n), (s))
 #  define mem_strdup(s)         xstrdup ((s))
 #  define mem_free(p)           xfree ((p))
-#  define mem_stats()        
+#  define mem_stats()
 
 #endif /* CALCURSE_MEMORY_DEBUG */
 
@@ -769,7 +751,7 @@ void      notify_update_bar (void);
 unsigned  notify_get_next (struct notify_app *);
 unsigned  notify_get_next_bkgd (void);
 char     *notify_app_txt (void);
-void      notify_check_next_app (void);
+void      notify_check_next_app (int);
 void      notify_check_added (char *, long, char);
 void      notify_check_repeated (struct recur_apoint *);
 int       notify_same_item (long);
@@ -777,34 +759,35 @@ int       notify_same_recur_item (struct recur_apoint *);
 void      notify_config_bar (void);
 
 /* recur.c */
-extern struct recur_apoint_list  *recur_alist_p;
-extern struct recur_event        *recur_elist;
+extern llist_ts_t recur_alist_p;
+extern llist_t recur_elist;
 void                  recur_event_free_bkp (enum eraseflg);
 void                  recur_apoint_free_bkp (enum eraseflg);
 void                  recur_apoint_llist_init (void);
 void                  recur_apoint_llist_free (void);
 void                  recur_event_llist_free (void);
 struct recur_apoint  *recur_apoint_new (char *, char *, long, long, char,
-                                       int, int, long, struct days **);
+                                       int, int, long, llist_t *);
 struct recur_event   *recur_event_new (char *, char *, long, int, int, int,
-                                       long, struct days **);
+                                       long, llist_t *);
 char                  recur_def2char (enum recur_type);
 int                   recur_char2def (char);
 struct recur_apoint *recur_apoint_scan (FILE *, struct tm, struct tm,
                                         char, int, struct tm, char *,
-                                        struct days **, char);
+                                        llist_t *, char);
 struct recur_event  *recur_event_scan (FILE *, struct tm, int, char,
                                        int, struct tm, char *,
-                                       struct days **);
+                                       llist_t *);
 void                 recur_save_data (FILE *);
-unsigned             recur_item_inday (long, struct days *, int, int,
-                                       long, long);
+unsigned             recur_item_inday (long, llist_t *, int, int, long, long);
+unsigned             recur_apoint_inday(struct recur_apoint *, long);
+unsigned             recur_event_inday(struct recur_event *, long);
 void                 recur_event_erase (long, unsigned, unsigned,
                                         enum eraseflg);
 void                 recur_apoint_erase (long, unsigned, unsigned,
                                          enum eraseflg);
 void                 recur_repeat_item (struct conf *);
-struct days         *recur_exc_scan (FILE *);
+void                 recur_exc_scan (llist_t *, FILE *);
 struct notify_app   *recur_apoint_check_next (struct notify_app *, long, long);
 struct recur_apoint *recur_get_apoint (long, int);
 struct recur_event  *recur_get_event (long, int);
@@ -817,7 +800,7 @@ void      sigs_init (void);
 unsigned  sigs_set_hdlr (int, void (*)(int));
 
 /* todo.c */
-extern struct todo *todolist;
+extern llist_t todolist;
 void          todo_hilt_set (int);
 void          todo_hilt_decrease (void);
 void          todo_hilt_increase (void);
@@ -838,10 +821,11 @@ void          todo_edit_item (void);
 void          todo_update_panel (int);
 void          todo_edit_note (char *);
 void          todo_view_note (char *);
+void          todo_init_list (void);
 void          todo_free_list (void);
 
 /* utils.c */
-void         exit_calcurse (int);
+void         exit_calcurse (int) __attribute__((__noreturn__));
 void         free_user_data (void);
 void         fatalbox (const char *);
 void         warnbox (const char *);
@@ -856,7 +840,6 @@ long         get_item_time (long);
 int          get_item_hour (long);
 int          get_item_min (long);
 long         date2sec (struct date, unsigned, unsigned);
-char        *date_sec2hour_str (long);
 char        *date_sec2date_str (long, char *);
 void         date_sec2date_fmt (long, const char *, char *);
 long         date_sec_change (long, int, int);
@@ -871,15 +854,18 @@ long         now (void);
 char        *nowstr (void);
 long         mystrtol (const char *);
 void         print_bool_option_incolor (WINDOW *, unsigned, int, int);
+const char  *get_tempdir (void);
 char        *new_tempfile (const char *, int);
 void         erase_note (char **, enum eraseflg);
-int          parse_date (char *, enum datefmt, int *, int *, int *);
-char        *str_toupper (char *);
+int          parse_date (char *, enum datefmt, int *, int *, int *,
+                         struct date *);
+void         str_toupper (char *);
 void         file_close (FILE *, const char *);
 void         psleep (unsigned);
 
 /* vars.c */
 extern int               col, row;
+extern int               resize;
 extern unsigned          colorize;
 extern enum ui_mode      ui_mode;
 extern int               days[12];
