@@ -1,9 +1,7 @@
-/*	$calcurse: day.c,v 1.53 2010-03-20 10:54:44 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -52,9 +50,21 @@ struct day_saved_item {
   char *mesg;
 };
 
-static struct day_item        *day_items_ptr;
+static llist_t                 day_items;
 static struct day_saved_item   day_saved_item;
 
+static void
+day_free (struct day_item *day)
+{
+  mem_free (day);
+}
+
+static void
+day_init_list (void)
+{
+  LLIST_INIT (&day_items);
+}
+
 /*
  * Free the current day linked list containing the events and appointments.
  * Must not free associated message and note, because their are not dynamically
@@ -63,205 +73,173 @@ static struct day_saved_item   day_saved_item;
 void
 day_free_list (void)
 {
-  struct day_item *o, **i;
-
-  i = &day_items_ptr;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o);
-    }
-  day_items_ptr = 0;
+  LLIST_FREE_INNER (&day_items, day_free);
+  LLIST_FREE (&day_items);
 }
 
 /* Add an event in the current day list */
 static struct day_item *
-day_add_event (int type, char *mesg, char *note, long day, int id)
+day_add_event (int type, char *mesg, char *note, long nday, int id)
 {
-  struct day_item *o, **i;
-  
-  o = mem_malloc (sizeof (struct day_item));
-  o->mesg = mesg;
-  o->note = note;
-  o->type = type;
-  o->appt_dur = 0;
-  o->appt_pos = 0;
-  o->start = day;
-  o->evnt_id = id;
-  i = &day_items_ptr;
-  for (;;)
+  struct day_item *day;
+
+  day = mem_malloc (sizeof (struct day_item));
+  day->mesg = mesg;
+  day->note = note;
+  day->type = type;
+  day->appt_dur = 0;
+  day->appt_pos = 0;
+  day->start = nday;
+  day->evnt_id = id;
+
+  LLIST_ADD (&day_items, day);
+
+  return day;
+}
+
+static int
+day_cmp_start (struct day_item *a, struct day_item *b)
+{
+  if (a->type <= EVNT)
     {
-      if (*i == 0)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
+      if (b->type <= EVNT)
+        return 0;
+      else
+        return -1;
     }
-  return (o);
+  else if (b->type <= EVNT)
+    return 1;
+  else
+    return (a->start < b->start ? -1 : (a->start == b->start ? 0 : 1));
 }
 
 /* Add an appointment in the current day list. */
 static struct day_item *
 day_add_apoint (int type, char *mesg, char *note, long start, long dur,
-		char state, int real_pos)
+                char state, int real_pos)
 {
-  struct day_item *o, **i;
-  int insert_item = 0;
+  struct day_item *day;
 
-  o = mem_malloc (sizeof (struct day_item));
-  o->mesg = mesg;
-  o->note = note;
-  o->start = start;
-  o->appt_dur = dur;
-  o->appt_pos = real_pos;
-  o->state = state;
-  o->type = type;
-  o->evnt_id = 0;
-  i = &day_items_ptr;
-  for (;;)
-    {
-      if (*i == 0)
-	{
-	  insert_item = 1;
-	}
-      else if (((*i)->start > start) && ((*i)->type > EVNT))
-	{
-	  insert_item = 1;
-	}
-      if (insert_item)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
-    }
-  return o;
+  day = mem_malloc (sizeof (struct day_item));
+  day->mesg = mesg;
+  day->note = note;
+  day->start = start;
+  day->appt_dur = dur;
+  day->appt_pos = real_pos;
+  day->state = state;
+  day->type = type;
+  day->evnt_id = 0;
+
+  LLIST_ADD_SORTED (&day_items, day, day_cmp_start);
+
+  return day;
 }
 
-/* 
+/*
  * Store the events for the selected day in structure pointed
- * by day_items_ptr. This is done by copying the events 
+ * by day_items. This is done by copying the events
  * from the general structure pointed by eventlist to the structure
- * dedicated to the selected day. 
+ * dedicated to the selected day.
  * Returns the number of events for the selected day.
  */
 static int
 day_store_events (long date)
 {
-  struct event *j;
-  struct day_item *ptr;
+  llist_item_t *i;
   int e_nb = 0;
 
-  for (j = eventlist; j != 0; j = j->next)
+  LLIST_FIND_FOREACH (&eventlist, date, event_inday, i)
     {
-      if (event_inday (j, date))
-	{
-	  e_nb++;
-	  ptr = day_add_event (EVNT, j->mesg, j->note, j->day, j->id);
-	}
+      struct event *ev = LLIST_TS_GET_DATA (i);
+      (void)day_add_event (EVNT, ev->mesg, ev->note, ev->day, ev->id);
+      e_nb++;
     }
 
   return e_nb;
 }
 
-/* 
+/*
  * Store the recurrent events for the selected day in structure pointed
- * by day_items_ptr. This is done by copying the recurrent events 
+ * by day_items. This is done by copying the recurrent events
  * from the general structure pointed by recur_elist to the structure
- * dedicated to the selected day. 
+ * dedicated to the selected day.
  * Returns the number of recurrent events for the selected day.
  */
 static int
 day_store_recur_events (long date)
 {
-  struct recur_event *j;
-  struct day_item *ptr;
+  llist_item_t *i;
   int e_nb = 0;
 
-  for (j = recur_elist; j != 0; j = j->next)
+  LLIST_FIND_FOREACH (&recur_elist, date, recur_event_inday, i)
     {
-      if (recur_item_inday (j->day, j->exc, j->rpt->type, j->rpt->freq,
-			    j->rpt->until, date))
-	{
-	  e_nb++;
-	  ptr = day_add_event (RECUR_EVNT, j->mesg, j->note, j->day, j->id);
-	}
+      struct recur_event *rev = LLIST_TS_GET_DATA (i);
+      (void)day_add_event (RECUR_EVNT, rev->mesg, rev->note, rev->day,
+                           rev->id);
+      e_nb++;
     }
 
   return e_nb;
 }
 
-/* 
+/*
  * Store the apoints for the selected day in structure pointed
- * by day_items_ptr. This is done by copying the appointments
- * from the general structure pointed by alist_p->root to the 
- * structure dedicated to the selected day. 
+ * by day_items. This is done by copying the appointments
+ * from the general structure pointed by alist_p to the
+ * structure dedicated to the selected day.
  * Returns the number of appointments for the selected day.
  */
 static int
 day_store_apoints (long date)
 {
-  struct apoint *j;
-  struct day_item *ptr;
+  llist_item_t *i;
   int a_nb = 0;
 
-  pthread_mutex_lock (&(alist_p->mutex));
-  for (j = alist_p->root; j != 0; j = j->next)
+  LLIST_TS_LOCK (&alist_p);
+  LLIST_TS_FIND_FOREACH (&alist_p, date, apoint_inday, i)
     {
-      if (apoint_inday (j, date))
-	{
-	  a_nb++;
-	  ptr = day_add_apoint (APPT, j->mesg, j->note, j->start,
-				j->dur, j->state, 0);
-	}
+      struct apoint *apt = LLIST_TS_GET_DATA (i);
+      (void)day_add_apoint (APPT, apt->mesg, apt->note, apt->start, apt->dur,
+                            apt->state, 0);
+      a_nb++;
     }
-  pthread_mutex_unlock (&(alist_p->mutex));
+  LLIST_TS_UNLOCK (&alist_p);
 
   return a_nb;
 }
 
-/* 
+/*
  * Store the recurrent apoints for the selected day in structure pointed
- * by day_items_ptr. This is done by copying the appointments
- * from the general structure pointed by recur_alist_p->root to the 
- * structure dedicated to the selected day. 
+ * by day_items. This is done by copying the appointments
+ * from the general structure pointed by recur_alist_p to the
+ * structure dedicated to the selected day.
  * Returns the number of recurrent appointments for the selected day.
  */
 static int
 day_store_recur_apoints (long date)
 {
-  struct recur_apoint *j;
-  struct day_item *ptr;
-  long real_start;
-  int a_nb = 0, n = 0;
+  llist_item_t *i;
+  int a_nb = 0;
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (j = recur_alist_p->root; j != 0; j = j->next)
+  LLIST_TS_LOCK (&recur_alist_p);
+  LLIST_TS_FIND_FOREACH (&recur_alist_p, date, recur_apoint_inday, i)
     {
-      if ((real_start = recur_item_inday (j->start, j->exc,
-					  j->rpt->type, j->rpt->freq,
-					  j->rpt->until, date)))
-	{
-	  a_nb++;
-	  ptr = day_add_apoint (RECUR_APPT, j->mesg, j->note,
-				real_start, j->dur, j->state, n);
-	  n++;
-	}
+      struct recur_apoint *rapt = LLIST_TS_GET_DATA (i);
+      int real_start = recur_apoint_inday (rapt, date);
+      (void)day_add_apoint (RECUR_APPT, rapt->mesg, rapt->note, real_start,
+                            rapt->dur, rapt->state, a_nb);
+      a_nb++;
     }
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+  LLIST_TS_UNLOCK (&recur_alist_p);
 
   return a_nb;
 }
 
-/* 
+/*
  * Store all of the items to be displayed for the selected day.
- * Items are of four types: recursive events, normal events, 
+ * Items are of four types: recursive events, normal events,
  * recursive appointments and normal appointments.
- * The items are stored in the linked list pointed by *day_items_ptr
+ * The items are stored in the linked list pointed by day_items
  * and the length of the new pad to write is returned.
  * The number of events and appointments in the current day are also updated.
  */
@@ -272,11 +250,8 @@ day_store_items (long date, unsigned *pnb_events, unsigned *pnb_apoints)
   int nb_events, nb_recur_events;
   int nb_apoints, nb_recur_apoints;
 
-  pad_length = nb_events = nb_apoints = 0;
-  nb_recur_events = nb_recur_apoints = 0;
-
-  if (day_items_ptr != 0)
-    day_free_list ();
+  day_free_list ();
+  day_init_list ();
   nb_recur_events = day_store_recur_events (date);
   nb_events = day_store_events (date);
   *pnb_events = nb_events;
@@ -298,7 +273,7 @@ day_store_items (long date, unsigned *pnb_events, unsigned *pnb_apoints)
  */
 struct day_items_nb *
 day_process_storage (struct date *slctd_date, unsigned day_changed,
-		     struct day_items_nb *inday)
+                     struct day_items_nb *inday)
 {
   long date;
   struct date day;
@@ -326,8 +301,8 @@ day_process_storage (struct date *slctd_date, unsigned day_changed,
 }
 
 /*
- * Returns a structure of type apoint_llist_node_t given a structure of type 
- * day_item_s 
+ * Returns a structure of type apoint_llist_node_t given a structure of type
+ * day_item_s
  */
 static void
 day_item_s2apoint_s (struct apoint *a, struct day_item *p)
@@ -338,12 +313,12 @@ day_item_s2apoint_s (struct apoint *a, struct day_item *p)
   a->mesg = p->mesg;
 }
 
-/* 
+/*
  * Print an item date in the appointment panel.
  */
 static void
 display_item_date (int incolor, struct apoint *i, int type, long date,
-		   int y, int x)
+                   int y, int x)
 {
   WINDOW *win;
   char a_st[100], a_end[100];
@@ -368,17 +343,20 @@ display_item_date (int incolor, struct apoint *i, int type, long date,
     custom_remove_attr (win, ATTR_HIGHEST);
 }
 
-/* 
+/*
  * Print an item description in the corresponding panel window.
  */
 static void
 display_item (int incolor, char *msg, int recur, int note, int len, int y,
-	      int x)
+              int x)
 {
   WINDOW *win;
   int ch_recur, ch_note;
   char buf[len];
 
+  if (len <= 0)
+    return;
+
   win = apad.ptrwin;
   ch_recur = (recur) ? '*' : ' ';
   ch_note = (note) ? '>' : ' ';
@@ -396,7 +374,7 @@ display_item (int incolor, char *msg, int recur, int note, int len, int y,
     custom_remove_attr (win, ATTR_HIGHEST);
 }
 
-/* 
+/*
  * Write the appointments and events for the selected day in a pad.
  * An horizontal line is drawn between events and appointments, and the
  * item selected by user is highlighted. This item is also saved inside
@@ -406,61 +384,61 @@ display_item (int incolor, char *msg, int recur, int note, int len, int y,
 void
 day_write_pad (long date, int width, int length, int incolor)
 {
-  struct day_item *p;
+  llist_item_t *i;
   struct apoint a;
-  int line, item_number, max_pos, recur;
+  int line, item_number, recur;
   const int x_pos = 0;
   unsigned draw_line = 0;
 
   line = item_number = 0;
-  max_pos = length;
 
-  for (p = day_items_ptr; p != 0; p = p->next)
+  LLIST_FOREACH (&day_items, i)
     {
-      if (p->type == RECUR_EVNT || p->type == RECUR_APPT)
-	recur = 1;
+      struct day_item *day = LLIST_TS_GET_DATA (i);
+      if (day->type == RECUR_EVNT || day->type == RECUR_APPT)
+        recur = 1;
       else
-	recur = 0;
+        recur = 0;
       /* First print the events for current day. */
-      if (p->type < RECUR_APPT)
-	{
-	  item_number++;
-	  if (item_number - incolor == 0)
-	    {
-	      day_saved_item.type = p->type;
-	      day_saved_item.mesg = p->mesg;
-	    }
-	  display_item (item_number - incolor, p->mesg, recur,
-			(p->note != NULL) ? 1 : 0, width - 7, line, x_pos);
-	  line++;
-	  draw_line = 1;
-	}
+      if (day->type < RECUR_APPT)
+        {
+          item_number++;
+          if (item_number - incolor == 0)
+            {
+              day_saved_item.type = day->type;
+              day_saved_item.mesg = day->mesg;
+            }
+          display_item (item_number - incolor, day->mesg, recur,
+                        (day->note != NULL) ? 1 : 0, width - 7, line, x_pos);
+          line++;
+          draw_line = 1;
+        }
       else
-	{
-	  /* Draw a line between events and appointments. */
-	  if (line > 0 && draw_line)
-	    {
-	      wmove (apad.ptrwin, line, 0);
-	      whline (apad.ptrwin, 0, width);
-	      draw_line = 0;
-	    }
-	  /* Last print the appointments for current day. */
-	  item_number++;
-	  day_item_s2apoint_s (&a, p);
-	  if (item_number - incolor == 0)
-	    {
-	      day_saved_item.type = p->type;
-	      day_saved_item.mesg = p->mesg;
-	      apoint_sec2str (&a, p->type, date,
-			      day_saved_item.start, day_saved_item.end);
-	    }
-	  display_item_date (item_number - incolor, &a, p->type,
-			     date, line + 1, x_pos);
-	  display_item (item_number - incolor, p->mesg, 0,
-			(p->note != NULL) ? 1 : 0, width - 7, line + 2,
-			x_pos);
-	  line += 3;
-	}
+        {
+          /* Draw a line between events and appointments. */
+          if (line > 0 && draw_line)
+            {
+              wmove (apad.ptrwin, line, 0);
+              whline (apad.ptrwin, 0, width);
+              draw_line = 0;
+            }
+          /* Last print the appointments for current day. */
+          item_number++;
+          day_item_s2apoint_s (&a, day);
+          if (item_number - incolor == 0)
+            {
+              day_saved_item.type = day->type;
+              day_saved_item.mesg = day->mesg;
+              apoint_sec2str (&a, day->type, date,
+                              day_saved_item.start, day_saved_item.end);
+            }
+          display_item_date (item_number - incolor, &a, day->type,
+                             date, line + 1, x_pos);
+          display_item (item_number - incolor, day->mesg, 0,
+                        (day->note != NULL) ? 1 : 0, width - 7, line + 2,
+                        x_pos);
+          line += 3;
+        }
     }
 }
 
@@ -472,52 +450,42 @@ day_popup_item (void)
     item_in_popup (NULL, NULL, day_saved_item.mesg, _("Event :"));
   else if (day_saved_item.type == APPT || day_saved_item.type == RECUR_APPT)
     item_in_popup (day_saved_item.start, day_saved_item.end,
-		   day_saved_item.mesg, _("Appointment :"));
+                   day_saved_item.mesg, _("Appointment :"));
   else
     EXIT (_("unknown item type"));
   /* NOTREACHED */
 }
 
-/* 
+/*
  * Need to know if there is an item for the current selected day inside
  * calendar. This is used to put the correct colors inside calendar panel.
  */
 int
 day_check_if_item (struct date day)
 {
-  struct recur_event *re;
-  struct recur_apoint *ra;
-  struct event *e;
-  struct apoint *a;
   const long date = date2sec (day, 0, 0);
 
-  for (re = recur_elist; re != 0; re = re->next)
-    if (recur_item_inday (re->day, re->exc, re->rpt->type,
-			  re->rpt->freq, re->rpt->until, date))
+  if (LLIST_FIND_FIRST (&recur_elist, date, recur_event_inday))
+    return (1);
+
+  LLIST_TS_LOCK (&recur_alist_p);
+  if (LLIST_TS_FIND_FIRST (&recur_alist_p, date, recur_apoint_inday))
+    {
+      LLIST_TS_UNLOCK (&recur_alist_p);
       return (1);
+    }
+  LLIST_TS_UNLOCK (&recur_alist_p);
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (ra = recur_alist_p->root; ra != 0; ra = ra->next)
-    if (recur_item_inday (ra->start, ra->exc, ra->rpt->type,
-			  ra->rpt->freq, ra->rpt->until, date))
-      {
-	pthread_mutex_unlock (&(recur_alist_p->mutex));
-	return (1);
-      }
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+  if (LLIST_FIND_FIRST (&eventlist, date, event_inday))
+    return (1);
 
-  for (e = eventlist; e != 0; e = e->next)
-    if (event_inday (e, date))
+  LLIST_TS_LOCK (&alist_p);
+  if (LLIST_TS_FIND_FIRST (&alist_p, date, apoint_inday))
+    {
+      LLIST_TS_UNLOCK (&alist_p);
       return (1);
-
-  pthread_mutex_lock (&(alist_p->mutex));
-  for (a = alist_p->root; a != 0; a = a->next)
-    if (apoint_inday (a, date))
-      {
-	pthread_mutex_unlock (&(alist_p->mutex));
-	return (1);
-      }
-  pthread_mutex_unlock (&(alist_p->mutex));
+    }
+  LLIST_TS_UNLOCK (&alist_p);
 
   return (0);
 }
@@ -529,13 +497,13 @@ fill_slices (int *slices, int slicesno, int first, int last)
 
   if (first < 0 || last < first)
     return 0;
-  
+
   if (last >= slicesno)
     last = slicesno - 1; /* Appointment spanning more than one day. */
-  
+
   for (i = first; i <= last; i++)
     slices[i] = 1;
-      
+
   return 1;
 }
 
@@ -547,49 +515,45 @@ fill_slices (int *slices, int slicesno, int first, int last)
 unsigned
 day_chk_busy_slices (struct date day, int slicesno, int *slices)
 {
-  struct recur_apoint *ra;
-  struct apoint *a;
-  int slicelen;  
+  llist_item_t *i;
+  int slicelen;
   const long date = date2sec (day, 0, 0);
 
   slicelen = DAYINSEC / slicesno;
 
 #define  SLICENUM(tsec)  ((tsec) / slicelen % slicesno)
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (ra = recur_alist_p->root; ra != 0; ra = ra->next)
-    if (recur_item_inday (ra->start, ra->exc, ra->rpt->type,
-			  ra->rpt->freq, ra->rpt->until, date))
-      {
-        long start, end;
+  LLIST_TS_LOCK (&recur_alist_p);
+  LLIST_TS_FIND_FOREACH (&recur_alist_p, date, recur_apoint_inday, i)
+    {
+      struct apoint *rapt = LLIST_TS_GET_DATA (i);
+      long start = get_item_time (rapt->start);
+      long end = get_item_time (rapt->start + rapt->dur);
 
-        start = get_item_time (ra->start);
-        end = get_item_time (ra->start + ra->dur);
-        if (!fill_slices (slices, slicesno, SLICENUM (start), SLICENUM (end)))
-          {
-            pthread_mutex_unlock (&(recur_alist_p->mutex));
-            return 0;
-          }
-      }
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+      if (!fill_slices (slices, slicesno, SLICENUM (start), SLICENUM (end)))
+        {
+          LLIST_TS_UNLOCK (&recur_alist_p);
+          return 0;
+        }
+    }
+  LLIST_TS_UNLOCK (&recur_alist_p);
 
-  pthread_mutex_lock (&(alist_p->mutex));
-  for (a = alist_p->root; a != 0; a = a->next)
-    if (apoint_inday (a, date))
-      {
-        long start, end;
+  LLIST_TS_LOCK (&alist_p);
+  LLIST_TS_FIND_FOREACH (&alist_p, date, apoint_inday, i)
+    {
+      struct apoint *apt = LLIST_TS_GET_DATA (i);
+      long start = get_item_time (apt->start);
+      long end = get_item_time (apt->start + apt->dur);
 
-        start = get_item_time (a->start);
-        end = get_item_time (a->start + a->dur);
-        if (!fill_slices (slices, slicesno, SLICENUM (start), SLICENUM (end)))
-          {
-            pthread_mutex_unlock (&(alist_p->mutex));
-            return 0;
-          }
-      }
-  pthread_mutex_unlock (&(alist_p->mutex));
+      if (!fill_slices (slices, slicesno, SLICENUM (start), SLICENUM (end)))
+        {
+          LLIST_TS_UNLOCK (&alist_p);
+          return 0;
+        }
+    }
+  LLIST_TS_UNLOCK (&alist_p);
 
-#undef SLICENUM  
+#undef SLICENUM
   return 1;
 }
 
@@ -605,15 +569,15 @@ day_edit_time (long time)
   while (1)
     {
       status_mesg (msg_time, "");
-      timestr = date_sec2hour_str (time);
+      timestr = date_sec2date_str (time, "%H:%M");
       updatestring (win[STA].p, &timestr, 0, 1);
       if (check_time (timestr) != 1 || strlen (timestr) == 0)
-	{
-	  status_mesg (fmt_msg, enter_str);
-	  (void)wgetch (win[STA].p);
-	}
+        {
+          status_mesg (fmt_msg, enter_str);
+          (void)wgetch (win[STA].p);
+        }
       else
-	return (timestr);
+        return (timestr);
     }
 }
 
@@ -634,17 +598,17 @@ update_start_time (long *start, long *dur)
       mem_free (timestr);
       newtime = update_time_in_date (*start, hr, mn);
       if (newtime < *start + *dur)
-	{
-	  *dur -= (newtime - *start);
-	  *start = newtime;
-	  valid_date = 1;
-	}
+        {
+          *dur -= (newtime - *start);
+          *start = newtime;
+          valid_date = 1;
+        }
       else
-	{
-	  status_mesg (msg_wrong_time, msg_enter);
-	  (void)wgetch (win[STA].p);
-	  valid_date = 0;
-	}
+        {
+          status_mesg (msg_wrong_time, msg_enter);
+          (void)wgetch (win[STA].p);
+          valid_date = 0;
+        }
     }
   while (valid_date == 0);
 }
@@ -695,15 +659,15 @@ update_rept (struct rpt **rpt, const long start, struct conf *conf)
       (void)snprintf (typstr, SINGLECHAR, "%c", recur_def2char ((*rpt)->type));
       cancel = updatestring (win[STA].p, &typstr, 0, 1);
       if (cancel)
-	{
-	  mem_free (typstr);
-	  return;
-	}
+        {
+          mem_free (typstr);
+          return;
+        }
       else
-	{
-	  ch = toupper (*typstr);
-	  mem_free (typstr);
-	}
+        {
+          ch = toupper (*typstr);
+          mem_free (typstr);
+        }
     }
   while ((ch != 'D') && (ch != 'W') && (ch != 'M') && (ch != 'Y'));
 
@@ -714,20 +678,20 @@ update_rept (struct rpt **rpt, const long start, struct conf *conf)
       (void)snprintf (freqstr, BUFSIZ, "%d", (*rpt)->freq);
       cancel = updatestring (win[STA].p, &freqstr, 0, 1);
       if (cancel)
-	{
-	  mem_free (freqstr);
-	  return;
-	}
+        {
+          mem_free (freqstr);
+          return;
+        }
       else
-	{
-	  newfreq = atoi (freqstr);
-	  mem_free (freqstr);
-	  if (newfreq == 0)
-	    {
-	      status_mesg (msg_wrong_freq, msg_enter);
-	      (void)wgetch (win[STA].p);
-	    }
-	}
+        {
+          newfreq = atoi (freqstr);
+          mem_free (freqstr);
+          if (newfreq == 0)
+            {
+              status_mesg (msg_wrong_freq, msg_enter);
+              (void)wgetch (win[STA].p);
+            }
+        }
     }
   while (newfreq == 0);
 
@@ -737,52 +701,52 @@ update_rept (struct rpt **rpt, const long start, struct conf *conf)
                       DATEFMT_DESC (conf->input_datefmt));
       status_mesg (_(outstr), "");
       timstr =
-	  date_sec2date_str ((*rpt)->until, DATEFMT (conf->input_datefmt));
+          date_sec2date_str ((*rpt)->until, DATEFMT (conf->input_datefmt));
       cancel = updatestring (win[STA].p, &timstr, 0, 1);
       if (cancel)
-	{
-	  mem_free (timstr);
-	  return;
-	}
+        {
+          mem_free (timstr);
+          return;
+        }
       if (strcmp (timstr, "0") == 0)
-	{
-	  newuntil = 0;
-	  date_entered = 1;
-	}
+        {
+          newuntil = 0;
+          date_entered = 1;
+        }
       else
-	{
-	  struct tm *lt;
-	  time_t t;
-	  struct date new_date;
-	  int newmonth, newday, newyear;
+        {
+          struct tm *lt;
+          time_t t;
+          struct date new_date;
+          int newmonth, newday, newyear;
 
-	  if (parse_date (timstr, conf->input_datefmt,
-			  &newyear, &newmonth, &newday))
-	    {
-	      t = start;
-	      lt = localtime (&t);
-	      new_date.dd = newday;
-	      new_date.mm = newmonth;
-	      new_date.yyyy = newyear;
-	      newuntil = date2sec (new_date, lt->tm_hour, lt->tm_min);
-	      if (newuntil < start)
-		{
-		  status_mesg (msg_wrong_time, msg_enter);
-		  (void)wgetch (win[STA].p);
-		  date_entered = 0;
-		}
-	      else
-		date_entered = 1;
-	    }
-	  else
-	    {
-	      (void)snprintf (outstr, BUFSIZ, msg_fmts,
+          if (parse_date (timstr, conf->input_datefmt,
+                          &newyear, &newmonth, &newday, calendar_get_slctd_day ()))
+            {
+              t = start;
+              lt = localtime (&t);
+              new_date.dd = newday;
+              new_date.mm = newmonth;
+              new_date.yyyy = newyear;
+              newuntil = date2sec (new_date, lt->tm_hour, lt->tm_min);
+              if (newuntil < start)
+                {
+                  status_mesg (msg_wrong_time, msg_enter);
+                  (void)wgetch (win[STA].p);
+                  date_entered = 0;
+                }
+              else
+                date_entered = 1;
+            }
+          else
+            {
+              (void)snprintf (outstr, BUFSIZ, msg_fmts,
                               DATEFMT_DESC (conf->input_datefmt));
-	      status_mesg (msg_wrong_date, _(outstr));
-	      (void)wgetch (win[STA].p);
-	      date_entered = 0;
-	    }
-	}
+              status_mesg (msg_wrong_date, _(outstr));
+              (void)wgetch (win[STA].p);
+              date_entered = 0;
+            }
+        }
     }
   while (date_entered == 0);
 
@@ -808,6 +772,7 @@ day_edit_item (struct conf *conf)
   struct apoint *a;
   long date;
   int item_num, ch;
+  int need_check_notify = 0;
 
   item_num = apoint_hilt ();
   p = day_get_item (item_num);
@@ -820,18 +785,18 @@ day_edit_item (struct conf *conf)
       re = recur_get_event (date, day_item_nb (date, item_num, RECUR_EVNT));
       status_mesg (_("Edit: (1)Description or (2)Repetition?"), "[1/2] ");
       while (ch != '1' && ch != '2' && ch != KEY_GENERIC_CANCEL)
-	ch = wgetch (win[STA].p);
+        ch = wgetch (win[STA].p);
       switch (ch)
-	{
-	case '1':
-	  update_desc (&re->mesg);
-	  break;
-	case '2':
-	  update_rept (&re->rpt, re->day, conf);
-	  break;
-	default:
-	  return;
-	}
+        {
+        case '1':
+          update_desc (&re->mesg);
+          break;
+        case '2':
+          update_rept (&re->rpt, re->day, conf);
+          break;
+        default:
+          return;
+        }
       break;
     case EVNT:
       e = event_get (date, day_item_nb (date, item_num, EVNT));
@@ -840,55 +805,65 @@ day_edit_item (struct conf *conf)
     case RECUR_APPT:
       ra = recur_get_apoint (date, day_item_nb (date, item_num, RECUR_APPT));
       status_mesg (_("Edit: (1)Start time, (2)End time, "
-		     "(3)Description or (4)Repetition?"), "[1/2/3/4] ");
+                     "(3)Description or (4)Repetition?"), "[1/2/3/4] ");
       while (ch != STRT && ch != END && ch != DESC &&
-	     ch != REPT && ch != KEY_GENERIC_CANCEL)
-	ch = wgetch (win[STA].p);
+             ch != REPT && ch != KEY_GENERIC_CANCEL)
+        ch = wgetch (win[STA].p);
       switch (ch)
-	{
-	case STRT:
-	  update_start_time (&ra->start, &ra->dur);
-	  break;
-	case END:
-	  update_duration (&ra->start, &ra->dur);
-	  break;
-	case DESC:
-	  update_desc (&ra->mesg);
-	  break;
-	case REPT:
-	  update_rept (&ra->rpt, ra->start, conf);
-	  break;
-	case KEY_GENERIC_CANCEL:
-	  return;
-	}
+        {
+        case STRT:
+          need_check_notify = 1;
+          update_start_time (&ra->start, &ra->dur);
+          break;
+        case END:
+          update_duration (&ra->start, &ra->dur);
+          break;
+        case DESC:
+          if (notify_bar ())
+            need_check_notify = notify_same_recur_item (ra);
+          update_desc (&ra->mesg);
+          break;
+        case REPT:
+          need_check_notify = 1;
+          update_rept (&ra->rpt, ra->start, conf);
+          break;
+        case KEY_GENERIC_CANCEL:
+          return;
+        }
       break;
     case APPT:
       a = apoint_get (date, day_item_nb (date, item_num, APPT));
       status_mesg (_("Edit: (1)Start time, (2)End time "
-		     "or (3)Description?"), "[1/2/3] ");
+                     "or (3)Description?"), "[1/2/3] ");
       while (ch != STRT && ch != END && ch != DESC && ch != KEY_GENERIC_CANCEL)
-	ch = wgetch (win[STA].p);
+        ch = wgetch (win[STA].p);
       switch (ch)
-	{
-	case STRT:
-	  update_start_time (&a->start, &a->dur);
-	  break;
-	case END:
-	  update_duration (&a->start, &a->dur);
-	  break;
-	case DESC:
-	  update_desc (&a->mesg);
-	  break;
-	case KEY_GENERIC_CANCEL:
-	  return;
-	}
+        {
+        case STRT:
+          need_check_notify = 1;
+          update_start_time (&a->start, &a->dur);
+          break;
+        case END:
+          update_duration (&a->start, &a->dur);
+          break;
+        case DESC:
+          if (notify_bar ())
+            need_check_notify = notify_same_item (a->start);
+          update_desc (&a->mesg);
+          break;
+        case KEY_GENERIC_CANCEL:
+          return;
+        }
       break;
     }
+
+  if (need_check_notify)
+    notify_check_next_app (1);
 }
 
 /*
  * In order to erase an item, we need to count first the number of
- * items for each type (in order: recurrent events, events, 
+ * items for each type (in order: recurrent events, events,
  * recurrent appointments and appointments) and then to test the
  * type of the item to be deleted.
  */
@@ -898,10 +873,10 @@ day_erase_item (long date, int item_number, enum eraseflg flag)
   struct day_item *p;
   char *erase_warning =
       _("This item is recurrent. "
-	"Delete (a)ll occurences or just this (o)ne ?");
+        "Delete (a)ll occurences or just this (o)ne ?");
   char *note_warning =
       _("This item has a note attached to it. "
-	"Delete (i)tem or just its (n)ote ?");
+        "Delete (i)tem or just its (n)ote ?");
   char *note_choice = _("[i/n] ");
   char *erase_choice = _("[a/o] ");
   int ch, ans;
@@ -913,16 +888,16 @@ day_erase_item (long date, int item_number, enum eraseflg flag)
     {
       ans = 0;
       if (p->note == NULL)
-	ans = 'i';
+        ans = 'i';
       while (ans != 'i' && ans != 'n')
-	{
-	  status_mesg (note_warning, note_choice);
-	  ans = wgetch (win[STA].p);
-	}
+        {
+          status_mesg (note_warning, note_choice);
+          ans = wgetch (win[STA].p);
+        }
       if (ans == 'i')
-	flag = ERASE_FORCE;
+        flag = ERASE_FORCE;
       else
-	flag = ERASE_FORCE_ONLY_NOTE;
+        flag = ERASE_FORCE_ONLY_NOTE;
     }
   if (p->type == EVNT)
     {
@@ -935,33 +910,33 @@ day_erase_item (long date, int item_number, enum eraseflg flag)
   else
     {
       if (flag == ERASE_FORCE_ONLY_NOTE)
-	ch = 'a';
+        ch = 'a';
       while ((ch != 'a') && (ch != 'o') && (ch != KEY_GENERIC_CANCEL))
-	{
-	  status_mesg (erase_warning, erase_choice);
-	  ch = wgetch (win[STA].p);
-	}
+        {
+          status_mesg (erase_warning, erase_choice);
+          ch = wgetch (win[STA].p);
+        }
       if (ch == 'a')
-	{
-	  delete_whole = 1;
-	}
+        {
+          delete_whole = 1;
+        }
       else if (ch == 'o')
-	{
-	  delete_whole = 0;
-	}
+        {
+          delete_whole = 0;
+        }
       else
-	{
-	  return (0);
-	}
+        {
+          return (0);
+        }
       if (p->type == RECUR_EVNT)
-	{
-	  recur_event_erase (date, day_item_nb (date, item_number, RECUR_EVNT),
-			     delete_whole, flag);
-	}
+        {
+          recur_event_erase (date, day_item_nb (date, item_number, RECUR_EVNT),
+                             delete_whole, flag);
+        }
       else
-	{
-	  recur_apoint_erase (date, p->appt_pos, delete_whole, flag);
-	}
+        {
+          recur_apoint_erase (date, p->appt_pos, delete_whole, flag);
+        }
     }
   if (flag == ERASE_FORCE_ONLY_NOTE)
     return 0;
@@ -975,13 +950,13 @@ day_cut_item (long date, int item_number)
 {
   const int DELETE_WHOLE = 1;
   struct day_item *p;
-  
+
   p = day_get_item (item_number);
   switch (p->type)
     {
     case EVNT:
       event_delete_bynum (date, day_item_nb (date, item_number, EVNT),
-                          ERASE_CUT);      
+                          ERASE_CUT);
       break;
     case RECUR_EVNT:
       recur_event_erase (date, day_item_nb (date, item_number, RECUR_EVNT),
@@ -1037,15 +1012,7 @@ day_paste_item (long date, int cut_item_type)
 struct day_item *
 day_get_item (int item_number)
 {
-  struct day_item *o;
-  int i;
-
-  o = day_items_ptr;
-  for (i = 1; i < item_number; i++)
-    {
-      o = o->next;
-    }
-  return (o);
+  return LLIST_GET_DATA (LLIST_NTH (&day_items, item_number - 1));
 }
 
 /* Returns the real item number, given its type. */
@@ -1053,17 +1020,17 @@ int
 day_item_nb (long date, int day_num, int type)
 {
   int i, nb_item[MAX_TYPES];
-  struct day_item *p;
+  llist_item_t *j;
 
   for (i = 0; i < MAX_TYPES; i++)
     nb_item[i] = 0;
 
-  p = day_items_ptr;
-
+  j = LLIST_FIRST (&day_items);
   for (i = 1; i < day_num; i++)
     {
-      nb_item[p->type - 1]++;
-      p = p->next;
+      struct day_item *day = LLIST_TS_GET_DATA (j);
+      nb_item[day->type - 1]++;
+      j = LLIST_TS_NEXT (j);
     }
 
   return (nb_item[type - 1]);
@@ -1088,13 +1055,16 @@ day_edit_note (char *editor)
   if (p->note == NULL)
     {
       if ((filename = new_tempfile (path_notes, NOTESIZ)) == NULL)
-	return;
+        return;
       else
-	p->note = filename;
+        p->note = filename;
     }
   (void)snprintf (fullname, BUFSIZ, "%s%s", path_notes, p->note);
   wins_launch_external (fullname, editor);
 
+  if (io_file_is_empty (fullname) > 0)
+    erase_note (&p->note, ERASE_FORCE);
+
   date = calendar_get_slctd_day_sec ();
   switch (p->type)
     {
