@@ -1,9 +1,7 @@
-/*	$calcurse: dmon.c,v 1.12 2010-03-20 10:54:44 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2009-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -71,7 +69,7 @@ dmon_sigs_hdlr (int sig)
 {
   if (data_loaded)
     free_user_data ();
-  
+
   DMON_LOG (_("terminated at %s with signal %d\n"), nowstr (), sig);
 
   if (unlink (path_dpid) != 0)
@@ -80,7 +78,7 @@ dmon_sigs_hdlr (int sig)
                 strerror (errno));
       exit (EXIT_FAILURE);
     }
-  
+
   exit (EXIT_SUCCESS);
 }
 
@@ -88,7 +86,7 @@ static unsigned
 daemonize (int status)
 {
   int fd;
-  
+
   /*
    * Operate in the background: Daemonizing.
    *
@@ -129,7 +127,7 @@ daemonize (int status)
                 strerror (errno));
       return 0;
     }
-      
+
   /* Redirect standard file descriptors to /dev/null. */
   if ((fd = open (_PATH_DEVNULL, O_RDWR, 0)) != -1)
     {
@@ -157,7 +155,7 @@ void
 dmon_start (int parent_exit_status)
 {
   struct conf conf;
-  
+
   if (!daemonize (parent_exit_status))
     DMON_ABRT (_("Cannot daemonize, aborting\n"));
 
@@ -168,12 +166,14 @@ dmon_start (int parent_exit_status)
     DMON_ABRT (_("Could not access \"%s\": %s\n"),
                path_conf, strerror (errno));
   custom_load_conf (&conf, 0);
-  
+
   if (!io_file_exist (path_apts))
     DMON_ABRT (_("Could not access \"%s\": %s\n"),
                path_apts, strerror (errno));
   apoint_llist_init ();
   recur_apoint_llist_init ();
+  event_llist_init ();
+  todo_init_list ();
   io_load_app ();
   data_loaded = 1;
 
@@ -181,7 +181,7 @@ dmon_start (int parent_exit_status)
   for (;;)
     {
       int left;
-      
+
       if (!notify_get_next_bkgd ())
         DMON_ABRT (_("error loading next appointment\n"));
 
@@ -193,7 +193,7 @@ dmon_start (int parent_exit_status)
           if (!notify_launch_cmd ())
             DMON_LOG (_("error while sending notification\n"));
         }
-      
+
       DMON_LOG (_("sleeping at %s for %d seconds\n"), nowstr (),
                 DMON_SLEEP_TIME);
       psleep (DMON_SLEEP_TIME);
@@ -214,6 +214,6 @@ dmon_stop (void)
   if (!dpid)
     return;
 
-  if (kill ((pid_t)dpid, SIGINT) < 0)
+  if (kill ((pid_t)dpid, SIGINT) < 0 && errno != ESRCH)
     EXIT (_("Could not stop calcurse daemon: %s\n"), strerror (errno));
 }
