@@ -1,9 +1,7 @@
-/*	$calcurse: keys.c,v 1.18 2010-03-20 10:54:46 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2008-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -80,9 +78,9 @@ static struct keydef_s keydef[NBKEYS] = {
   {"generic-next-week", "C-j"},
   {"generic-prev-week", "C-k"},
   {"generic-scroll-down", "C-n"},
-  {"generic-scroll-up", "C-p"},  
+  {"generic-scroll-up", "C-p"},
   {"generic-goto-today", "C-g"},
-        
+
   {"move-right", "l L"},
   {"move-left", "h H"},
   {"move-down", "j J"},
@@ -147,7 +145,7 @@ keys_free (void)
 
   for (key = 0; key < NBKEYS; key++)
     {
-      if (keys[key] == 0)
+      if (keys[key] == NULL)
         continue;
 
       i = &keys[key];
@@ -166,7 +164,7 @@ keys_dump_defaults (char *file)
 {
   FILE *fd;
   int i;
-  
+
   fd = fopen (file, "w");
   EXIT_IF (fd == NULL, _("FATAL ERROR: could not create default keys file."));
 
@@ -198,7 +196,7 @@ enum key
 keys_getch (WINDOW *win)
 {
   int ch;
-  
+
   ch = wgetch (win);
   switch (ch)
     {
@@ -213,7 +211,7 @@ static void
 add_key_str (enum key action, int key)
 {
   struct key_str_s *new, **i;
-  
+
   if (action < 0 || action > NBKEYS)
     return;
 
@@ -247,7 +245,7 @@ keys_assign_binding (int key, enum key action)
       actions[key] =  action;
       add_key_str (action, key);
     }
-  
+
   return 0;
 }
 
@@ -256,7 +254,7 @@ del_key_str (enum key action, int key)
 {
   struct key_str_s *old, **i;
   char oldstr[BUFSIZ];
-  
+
   if (action < 0 || action > NBKEYS)
     return;
 
@@ -267,7 +265,7 @@ del_key_str (enum key action, int key)
         {
           old = *i;
           *i = old->next;
-          mem_free (old->str);          
+          mem_free (old->str);
           mem_free (old);
           break;
         }
@@ -291,7 +289,7 @@ keys_str2int (char *key)
 {
   const struct string CONTROL_KEY = STRING_BUILD ("C-");
   const struct string TAB_KEY = STRING_BUILD ("TAB");
-  const struct string SPACE_KEY = STRING_BUILD ("SPC");  
+  const struct string SPACE_KEY = STRING_BUILD ("SPC");
   const struct string ESCAPE_KEY = STRING_BUILD ("ESC");
   const struct string CURSES_KEY_UP = STRING_BUILD ("UP");
   const struct string CURSES_KEY_DOWN = STRING_BUILD ("DWN");
@@ -377,7 +375,7 @@ keys_action_count_keys (enum key action)
 char *
 keys_action_firstkey (enum key action)
 {
-  return (keys[action] != NULL) ? keys[action]->str : "XXX"; 
+  return (keys[action] != NULL) ? keys[action]->str : "XXX";
 }
 
 char *
@@ -402,10 +400,10 @@ keys_action_allkeys (enum key action)
   static char keystr[BUFSIZ];
   struct key_str_s *i;
   const char *CHAR_SPACE = " ";
-  
+
   if (keys[action] == NULL)
     return NULL;
-  keystr[0] = '\0';  
+  keystr[0] = '\0';
   for (i = keys[action]; i; i = i->next)
     {
       const int MAXLEN = sizeof (keystr) - 1 - strlen (keystr);
@@ -456,7 +454,7 @@ keys_display_bindings_bar (WINDOW *win, struct binding **binding, int first_key,
   cmdlen =  KEYS_KEYLEN + 1 + KEYS_LABELEN;
   space_between_cmds = floor (col / KEYS_CMDS_PER_LINE - cmdlen);
   cmdlen += space_between_cmds;
-  
+
   j = 0;
   wins_erase_status_bar ();
   for (i = first_key; i < last_key; i += 2)
@@ -474,13 +472,14 @@ keys_display_bindings_bar (WINDOW *win, struct binding **binding, int first_key,
         {
           (void)strncpy (key, keys_action_firstkey (binding[i + 1]->action),
                          KEYS_KEYLEN);
+          key[KEYS_KEYLEN] = 0;
           fmtkey = keys_format_label (key, KEYS_KEYLEN);
           mvwprintw (win, 1, KEY_POS, fmtkey);
         }
       custom_remove_attr (win, ATTR_HIGHEST);
       mvwprintw (win, 0, LABEL_POS, binding[i]->label);
       if (i + 1 != last_key)
-	mvwprintw (win, 1, LABEL_POS, binding[i + 1]->label);
+        mvwprintw (win, 1, LABEL_POS, binding[i + 1]->label);
       j++;
     }
   wnoutrefresh (win);
@@ -495,7 +494,7 @@ keys_popup_info (enum key key)
 {
   char *info[NBKEYS];
   WINDOW *infowin;
-  
+
   info[KEY_GENERIC_CANCEL] =
     _("Cancel the ongoing action.");
   info[KEY_GENERIC_SELECT] =
@@ -575,8 +574,8 @@ keys_popup_info (enum key key)
   info[KEY_RAISE_PRIORITY] =
     _("Raise a task priority inside the todo panel.");
   info[KEY_LOWER_PRIORITY] =
-    _("Lower a task priority inside the todo panel.");    
-    
+    _("Lower a task priority inside the todo panel.");
+
   if (key < 0 || key > NBKEYS)
     return;
 
@@ -594,7 +593,7 @@ void
 keys_save_bindings (FILE *fd)
 {
   int i;
-  
+
   EXIT_IF (fd == NULL, _("FATAL ERROR: null file pointer."));
   dump_intro (fd);
   for (i = 0; i < NBKEYS; i++)
@@ -605,10 +604,10 @@ int
 keys_check_missing_bindings (void)
 {
   int i;
-    
+
   for (i = 0; i < NBKEYS; i++)
     {
-      if (keys[i] == 0)
+      if (keys[i] == NULL)
         return 1;
     }
   return 0;
@@ -621,11 +620,11 @@ keys_fill_missing (void)
 
   for (i = 0; i < NBKEYS; i++)
     {
-      if (keys[i] == 0)
+      if (keys[i] == NULL)
         {
           char *p, tmpbuf[BUFSIZ];
 
-          (void)strncpy (tmpbuf, keydef[i].binding, BUFSIZ);          
+          (void)strncpy (tmpbuf, keydef[i].binding, BUFSIZ);
           p = tmpbuf;
           for (;;)
             {
@@ -636,7 +635,7 @@ keys_fill_missing (void)
               if (sscanf (p, "%s", key_ch) == 1)
                 {
                   int ch, used;
-                  
+
                   ch = keys_str2int (key_ch);
                   used = keys_assign_binding (ch, i);
                   if (used)
