@@ -1,9 +1,7 @@
-/*	$calcurse: mem.c,v 1.7 2010-03-20 10:54:46 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2008-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -76,15 +74,14 @@ stats_add_blk (size_t size, const char *pos)
   struct mem_blk *o, **i;
 
   o = malloc (sizeof (*o));
-  EXIT_IF (o == 0, _("could not allocate memory to store block info"));
+  EXIT_IF (o == NULL, _("could not allocate memory to store block info"));
 
   mstats.ncall++;
-  
+
   o->pos = pos;
   o->size = (unsigned)size;
   o->next = 0;
-  
-  i = &mstats.blk;
+
   for (i = &mstats.blk; *i; i = &(*i)->next)
     ;
   o->id = mstats.ncall;
@@ -98,8 +95,6 @@ stats_del_blk (unsigned id)
 {
   struct mem_blk *o, **i;
 
-  EXIT_IF (id < 0, _("Incorrect block id"));
-
   i = &mstats.blk;
   for (o = mstats.blk; o; o = o->next)
     {
@@ -113,17 +108,17 @@ stats_del_blk (unsigned id)
     }
 
   EXIT (_("Block not found"));
-  /* NOTREACHED */  
+  /* NOTREACHED */
 }
 
 void *
 xmalloc (size_t size)
 {
   void *p;
-  
+
   EXIT_IF (size == 0, _("xmalloc: zero size"));
   p = malloc (size);
-  EXIT_IF (p == 0, _("xmalloc: out of memory"));
+  EXIT_IF (p == NULL, _("xmalloc: out of memory"));
 
   return p;
 }
@@ -136,7 +131,7 @@ xcalloc (size_t nmemb, size_t size)
   EXIT_IF (nmemb == 0 || size == 0, _("xcalloc: zero size"));
   EXIT_IF (SIZE_MAX / nmemb < size, _("xcalloc: overflow"));
   p = calloc (nmemb, size);
-  EXIT_IF (p == 0, _("xcalloc: out of memory"));
+  EXIT_IF (p == NULL, _("xcalloc: out of memory"));
 
   return p;
 }
@@ -151,7 +146,7 @@ xrealloc (void *ptr, size_t nmemb, size_t size)
   EXIT_IF (new_size == 0, _("xrealloc: zero size"));
   EXIT_IF (SIZE_MAX / nmemb < size, _("xrealloc: overflow"));
   new_ptr = realloc (ptr, new_size);
-  EXIT_IF (new_ptr == 0, _("xrealloc: out of memory"));
+  EXIT_IF (new_ptr == NULL, _("xrealloc: out of memory"));
 
   return new_ptr;
 }
@@ -171,7 +166,7 @@ xstrdup (const char *str)
 void
 xfree (void *p)
 {
-  EXIT_IF (p == 0, _("xfree: null pointer"));
+  EXIT_IF (p == NULL, _("xfree: null pointer"));
   free (p);
 }
 
@@ -179,13 +174,13 @@ void *
 dbg_malloc (size_t size, const char *pos)
 {
   unsigned *buf;
-  
+
    if (size == 0)
     return (void *)0;
-  
+
   size = EXTRA_SPACE + (size + sizeof (unsigned) - 1) / sizeof (unsigned);
   buf = xmalloc (size * sizeof (unsigned));
-  
+
   buf[BLK_STATE] = MAGIC_ALLOC;             /* state of the block */
   buf[BLK_SIZE] = size;                     /* size of the block */
   buf[BLK_ID] = stats_add_blk (size, pos);  /* identify a block by its id */
@@ -200,14 +195,14 @@ void *
 dbg_calloc (size_t nmemb, size_t size, const char *pos)
 {
   void *buf;
-  
+
   if (!nmemb || !size)
     return (void *)0;
-  
+
   EXIT_IF (nmemb > SIZE_MAX / size, _("overflow at %s"), pos);
 
   size *= nmemb;
-  if ((buf = dbg_malloc (size, pos)) == 0)
+  if ((buf = dbg_malloc (size, pos)) == NULL)
     return (void *)0;
 
   bzero (buf, size);
@@ -219,25 +214,25 @@ void *
 dbg_realloc (void *ptr, size_t nmemb, size_t size, const char *pos)
 {
   unsigned *buf, old_size, new_size, cpy_size;
-  
-  if (ptr == 0)
+
+  if (ptr == NULL)
     return (void *)0;
 
   new_size = nmemb *size;
   if (new_size == 0)
     return (void *)0;
-  
+
   EXIT_IF (nmemb > SIZE_MAX / size, _("overflow at %s"), pos);
-  
-  if ((buf = dbg_malloc (new_size, pos)) == 0)
+
+  if ((buf = dbg_malloc (new_size, pos)) == NULL)
     return (void *)0;
 
   old_size = *((unsigned *)ptr - EXTRA_SPACE_START + BLK_SIZE);
   cpy_size = (old_size > new_size) ? new_size : old_size;
   bcopy (ptr, buf, cpy_size);
-  
+
   mem_free (ptr);
-  
+
   return (void *)buf;
 }
 
@@ -246,12 +241,12 @@ dbg_strdup (const char *s, const char *pos)
 {
   size_t size;
   char *buf;
-  
-  if (s == 0)
+
+  if (s == NULL)
     return (char *)0;
 
   size = strlen (s);
-  if ((buf = dbg_malloc (size + 1, pos)) == 0)
+  if ((buf = dbg_malloc (size + 1, pos)) == NULL)
     return (char *)0;
 
   return strncpy (buf, s, size + 1);
@@ -262,23 +257,23 @@ dbg_free (void *ptr, const char *pos)
 {
   unsigned *buf, size;
 
-  EXIT_IF (ptr == 0, _("dbg_free: null pointer at %s"), pos);
+  EXIT_IF (ptr == NULL, _("dbg_free: null pointer at %s"), pos);
 
   buf = (unsigned *)ptr - EXTRA_SPACE_START;
   size = buf[BLK_SIZE];
-  
+
   EXIT_IF (buf[BLK_STATE] == MAGIC_FREE,
            _("block seems already freed at %s"), pos);
   EXIT_IF (buf[BLK_STATE] != MAGIC_ALLOC,
            _("corrupt block header at %s"), pos);
   EXIT_IF (buf[size - 1] != buf[BLK_ID],
            _("corrupt block end at %s, (end = %u, should be %d)"), pos,
-           buf[size - 1], buf[BLK_ID]); 
+           buf[size - 1], buf[BLK_ID]);
 
   buf[0] = MAGIC_FREE;
 
   stats_del_blk (buf[BLK_ID]);
-  
+
   free (buf);
   mstats.nfree += size;
 }
@@ -289,12 +284,12 @@ dbg_free (void *ptr, const char *pos)
 static void
 dump_block_info (struct mem_blk *blk)
 {
-  if (blk == 0)
+  if (blk == NULL)
     return;
-  
+
   printf (_("---==== MEMORY BLOCK ====----------------\n"));
-  printf (_("            id: %u\n"), blk->id);    
-  printf (_("          size: %u\n"), blk->size);  
+  printf (_("            id: %u\n"), blk->id);
+  printf (_("          size: %u\n"), blk->size);
   printf (_("  allocated in: %s\n"), blk->pos);
   printf (_("-----------------------------------------\n"));
 }
@@ -303,7 +298,7 @@ void
 mem_stats (void)
 {
   printf ("\n");
-  printf (_("+------------------------------+\n"));  
+  printf (_("+------------------------------+\n"));
   printf (_("| calcurse memory usage report |\n"));
   printf (_("+------------------------------+\n"));
   printf (_("  number of calls: %u\n"), mstats.ncall);
