@@ -1,9 +1,7 @@
-/*	$calcurse: recur.c,v 1.54 2010-03-20 10:54:47 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -44,53 +42,54 @@
 
 #include "calcurse.h"
 
-struct recur_apoint_list         *recur_alist_p;
-struct recur_event               *recur_elist;
-static struct recur_event         bkp_cut_recur_event;
-static struct recur_apoint        bkp_cut_recur_apoint;
-
+llist_ts_t                 recur_alist_p;
+llist_t                    recur_elist;
+static struct recur_event  bkp_cut_recur_event;
+static struct recur_apoint bkp_cut_recur_apoint;
 
 static void
-free_exc (struct days *exc)
+free_exc (struct excp *exc)
 {
-  struct days *o, **i;
-
-  i = &exc;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o);
-    }
+  mem_free (exc);
 }
 
 static void
-recur_add_exc (struct days **exc, long day)
+free_exc_list (llist_t *exc)
 {
-  struct days **i, *o;
-  
-  o = mem_malloc (sizeof (struct days));
+  LLIST_FREE_INNER (exc, free_exc);
+  LLIST_FREE (exc);
+}
+
+static int
+exc_cmp_day (struct excp *a, struct excp *b)
+{
+  return (a->st < b->st ? -1 : (a->st == b->st ? 0 : 1));
+}
+
+static void
+recur_add_exc (llist_t *exc, long day)
+{
+  struct excp *o = mem_malloc (sizeof (struct excp));
   o->st = day;
-  i = exc;
-  for (;;)
-    {
-      if (*i == 0 || (*i)->st > day)
-        {
-          o->next = *i;
-          *i = o;
-          break;
-        }
-      i = &(*i)->next;
-    }
+
+  LLIST_ADD_SORTED (exc, o, exc_cmp_day);
 }
 
 static void
-exc_dup (struct days **in, struct days *exc)
+exc_dup (llist_t *in, llist_t *exc)
 {
-  struct days *p;
-  
-  for (p = exc; p; p = p->next)
-    recur_add_exc (in, p->st);
+  llist_item_t *i;
+
+  LLIST_INIT (in);
+
+  if (exc)
+    {
+      LLIST_FOREACH (exc, i)
+        {
+          struct excp *p = LLIST_GET_DATA (i);
+          recur_add_exc (in, p->st);
+        }
+    }
 }
 
 void
@@ -106,11 +105,7 @@ recur_event_free_bkp (enum eraseflg flag)
       mem_free (bkp_cut_recur_event.rpt);
       bkp_cut_recur_event.rpt = 0;
     }
-  if (bkp_cut_recur_event.exc)
-    {
-      free_exc (bkp_cut_recur_event.exc);
-      bkp_cut_recur_event.exc = 0;
-    }
+  free_exc_list (&bkp_cut_recur_event.exc);
   erase_note (&bkp_cut_recur_event.note, flag);
 }
 
@@ -127,11 +122,7 @@ recur_apoint_free_bkp (enum eraseflg flag)
       mem_free (bkp_cut_recur_apoint.rpt);
       bkp_cut_recur_apoint.rpt = 0;
     }
-  if (bkp_cut_recur_apoint.exc)
-    {
-      free_exc (bkp_cut_recur_apoint.exc);
-      bkp_cut_recur_apoint.exc = 0;
-    }
+  free_exc_list (&bkp_cut_recur_apoint.exc);
   erase_note (&bkp_cut_recur_apoint.note, flag);
 }
 
@@ -143,14 +134,13 @@ recur_event_dup (struct recur_event *in, struct recur_event *bkp)
   bkp->id = in->id;
   bkp->day = in->day;
   bkp->mesg = mem_strdup (in->mesg);
-  
+
   bkp->rpt = mem_malloc (sizeof (struct rpt));
   bkp->rpt->type = in->rpt->type;
   bkp->rpt->freq = in->rpt->freq;
   bkp->rpt->until = in->rpt->until;
 
-  if (in->exc)
-    exc_dup (&bkp->exc, in->exc);
+  exc_dup (&bkp->exc, &in->exc);
 
   if (in->note)
     bkp->note = mem_strdup (in->note);
@@ -165,159 +155,136 @@ recur_apoint_dup (struct recur_apoint *in, struct recur_apoint *bkp)
   bkp->dur = in->dur;
   bkp->state = in->state;
   bkp->mesg = mem_strdup (in->mesg);
-  
+
   bkp->rpt = mem_malloc (sizeof (struct rpt));
   bkp->rpt->type = in->rpt->type;
   bkp->rpt->freq = in->rpt->freq;
   bkp->rpt->until = in->rpt->until;
 
-  if (in->exc)
-    exc_dup (&bkp->exc, in->exc);
+  exc_dup (&bkp->exc, &in->exc);
 
   if (in->note)
     bkp->note = mem_strdup (in->note);
-} 
+}
 
 void
 recur_apoint_llist_init (void)
 {
-  recur_alist_p = mem_malloc (sizeof (struct recur_apoint_list));
-  recur_alist_p->root = NULL;
-  pthread_mutex_init (&(recur_alist_p->mutex), NULL);
+  LLIST_TS_INIT (&recur_alist_p);
+}
+
+static void
+recur_apoint_free (struct recur_apoint *rapt)
+{
+  mem_free (rapt->mesg);
+  if (rapt->note)
+    mem_free (rapt->note);
+  if (rapt->rpt)
+    mem_free (rapt->rpt);
+  free_exc_list (&rapt->exc);
+  mem_free (rapt);
+}
+
+static void
+recur_event_free (struct recur_event *rev)
+{
+  mem_free (rev->mesg);
+  if (rev->note)
+    mem_free (rev->note);
+  if (rev->rpt)
+    mem_free (rev->rpt);
+  free_exc_list (&rev->exc);
+  mem_free (rev);
 }
 
 void
 recur_apoint_llist_free (void)
 {
-  struct recur_apoint *o, **i;
-
-  i = &recur_alist_p->root;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o->mesg);
-      if (o->note)
-        mem_free (o->note);
-      if (o->rpt)
-        mem_free (o->rpt);
-      if (o->exc)
-        {
-          free_exc (o->exc);
-          o->exc = 0;
-        }
-      mem_free (o);
-    }
-  mem_free (recur_alist_p);
+  LLIST_TS_FREE_INNER (&recur_alist_p, recur_apoint_free);
+  LLIST_TS_FREE (&recur_alist_p);
 }
 
 void
 recur_event_llist_free (void)
 {
-  struct recur_event *o, **i;
+  LLIST_FREE_INNER (&recur_elist, recur_event_free);
+  LLIST_FREE (&recur_elist);
+}
 
-  i = &recur_elist;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o->mesg);
-      if (o->note)
-        mem_free (o->note);
-      if (o->rpt)
-        mem_free (o->rpt);
-      if (o->exc)
-        {
-          free_exc (o->exc);
-          o->exc = 0;
-        }
-      mem_free (o);
-    }
+static int
+recur_apoint_cmp_start (struct recur_apoint *a, struct recur_apoint *b)
+{
+  return (a->start < b->start ? -1 : (a->start == b->start ? 0 : 1));
+}
+
+static int
+recur_event_cmp_day (struct recur_event *a, struct recur_event *b)
+{
+  return (a->day < b->day ? -1 : (a->day == b->day ? 0 : 1));
 }
 
 /* Insert a new recursive appointment in the general linked list */
 struct recur_apoint *
 recur_apoint_new (char *mesg, char *note, long start, long dur, char state,
-		  int type, int freq, long until, struct days **except)
+                  int type, int freq, long until, llist_t *except)
 {
-  struct recur_apoint *o, **i;
-  
-  o = mem_malloc (sizeof (struct recur_apoint));
-  o->rpt = mem_malloc (sizeof (struct rpt));
-  o->mesg = mem_strdup (mesg);
-  o->note = (note != 0) ? mem_strdup (note) : 0;
-  o->start = start;
-  o->state = state;
-  o->dur = dur;
-  o->rpt->type = type;
-  o->rpt->freq = freq;
-  o->rpt->until = until;
-  o->exc = 0;
-  if (except && *except)
-    {
-      exc_dup (&o->exc, *except);
-      free_exc (*except);
-      *except = 0;
-    }
+  struct recur_apoint *rapt = mem_malloc (sizeof (struct recur_apoint));
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  i = &recur_alist_p->root;
-  for (;;)
+  rapt->rpt = mem_malloc (sizeof (struct rpt));
+  rapt->mesg = mem_strdup (mesg);
+  rapt->note = (note != NULL) ? mem_strdup (note) : 0;
+  rapt->start = start;
+  rapt->state = state;
+  rapt->dur = dur;
+  rapt->rpt->type = type;
+  rapt->rpt->freq = freq;
+  rapt->rpt->until = until;
+  if (except)
     {
-      if (*i == 0 || (*i)->start > start)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
+      exc_dup (&rapt->exc, except);
+      free_exc_list (except);
     }
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+  else
+    LLIST_INIT (&rapt->exc);
 
-  return (o);
+  LLIST_TS_LOCK (&recur_alist_p);
+  LLIST_TS_ADD_SORTED (&recur_alist_p, rapt, recur_apoint_cmp_start);
+  LLIST_TS_UNLOCK (&recur_alist_p);
+
+  return rapt;
 }
 
 /* Insert a new recursive event in the general linked list */
 struct recur_event *
 recur_event_new (char *mesg, char *note, long day, int id, int type, int freq,
-		 long until, struct days **except)
+                 long until, llist_t *except)
 {
-  struct recur_event *o, **i;
-  
-  o = mem_malloc (sizeof (struct recur_event));
-  o->rpt = mem_malloc (sizeof (struct rpt));
-  o->mesg = mem_strdup (mesg);
-  o->note = (note != 0) ? mem_strdup (note) : 0;  
-  o->day = day;
-  o->id = id;
-  o->rpt->type = type;
-  o->rpt->freq = freq;
-  o->rpt->until = until;
-  o->exc = 0;
-  if (except && *except)
-    {
-      exc_dup (&o->exc, *except);
-      free_exc (*except);
-      *except = 0;
-    }
+  struct recur_event *rev = mem_malloc (sizeof (struct recur_event));
 
-  i = &recur_elist;
-  for (;;)
+  rev->rpt = mem_malloc (sizeof (struct rpt));
+  rev->mesg = mem_strdup (mesg);
+  rev->note = (note != NULL) ? mem_strdup (note) : 0;
+  rev->day = day;
+  rev->id = id;
+  rev->rpt->type = type;
+  rev->rpt->freq = freq;
+  rev->rpt->until = until;
+  if (except)
     {
-      if (*i == 0 || (*i)->day > day)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
+      exc_dup (&rev->exc, except);
+      free_exc_list (except);
     }
-  return (o);
+  else
+    LLIST_INIT (&rev->exc);
+
+  LLIST_ADD_SORTED (&recur_elist, rev, recur_event_cmp_day);
+
+  return rev;
 }
 
-/* 
- * Correspondance between the defines on recursive type, 
- * and the letter to be written in file. 
+/*
+ * Correspondance between the defines on recursive type,
+ * and the letter to be written in file.
  */
 char
 recur_def2char (enum recur_type define)
@@ -346,7 +313,7 @@ recur_def2char (enum recur_type define)
   return (recur_char);
 }
 
-/* 
+/*
  * Correspondance between the letters written in file and the defines
  * concerning the recursive type.
  */
@@ -378,36 +345,33 @@ recur_char2def (char type)
 
 /* Write days for which recurrent items should not be repeated. */
 static void
-recur_write_exc (struct days *exc, FILE *f)
+recur_write_exc (llist_t *lexc, FILE *f)
 {
+  llist_item_t *i;
   struct tm *lt;
   time_t t;
   int st_mon, st_day, st_year;
 
-  while (exc)
+  LLIST_FOREACH (lexc, i)
     {
+      struct excp *exc = LLIST_GET_DATA (i);
       t = exc->st;
       lt = localtime (&t);
       st_mon = lt->tm_mon + 1;
       st_day = lt->tm_mday;
       st_year = lt->tm_year + 1900;
       (void)fprintf (f, " !%02u/%02u/%04u", st_mon, st_day, st_year);
-      exc = exc->next;
     }
 }
 
 /* Load the recursive appointment description */
 struct recur_apoint *
 recur_apoint_scan (FILE *f, struct tm start, struct tm end, char type,
-		   int freq, struct tm until, char *note, struct days **exc,
-		   char state)
+                   int freq, struct tm until, char *note, llist_t *exc,
+                   char state)
 {
-  struct tm *lt;
   char buf[BUFSIZ], *nl;
-  time_t tstart, tend, t, tuntil;
-
-  t = time (NULL);
-  lt = localtime (&t);
+  time_t tstart, tend, tuntil;
 
   /* Read the appointment description */
   (void)fgets (buf, sizeof buf, f);
@@ -443,13 +407,13 @@ recur_apoint_scan (FILE *f, struct tm start, struct tm end, char type,
            _("date error in appointment"));
 
   return (recur_apoint_new (buf, note, tstart, tend - tstart, state,
-			    recur_char2def (type), freq, tuntil, exc));
+                            recur_char2def (type), freq, tuntil, exc));
 }
 
 /* Load the recursive events from file */
 struct recur_event *
 recur_event_scan (FILE *f, struct tm start, int id, char type, int freq,
-		  struct tm until, char *note, struct days **exc)
+                  struct tm until, char *note, llist_t *exc)
 {
   char buf[BUFSIZ], *nl;
   time_t tstart, tuntil;
@@ -516,8 +480,7 @@ recur_apoint_write (struct recur_apoint *o, FILE *f)
                      o->rpt->freq, recur_def2char (o->rpt->type),
                      lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year);
     }
-  if (o->exc != 0)
-    recur_write_exc (o->exc, f);
+  recur_write_exc (&o->exc, f);
   (void)fprintf (f, "} ");
   if (o->note != NULL)
     (void)fprintf (f, ">%s ", o->note);
@@ -560,8 +523,7 @@ recur_event_write (struct recur_event *o, FILE *f)
                      o->rpt->freq, recur_def2char (o->rpt->type),
                      end_mon, end_day, end_year);
     }
-  if (o->exc != 0)
-    recur_write_exc (o->exc, f);
+  recur_write_exc (&o->exc, f);
   (void)fprintf (f, "} ");
   if (o->note != NULL)
     (void)fprintf (f, ">%s ", o->note);
@@ -572,16 +534,21 @@ recur_event_write (struct recur_event *o, FILE *f)
 void
 recur_save_data (FILE *f)
 {
-  struct recur_event *re;
-  struct recur_apoint *ra;
+  llist_item_t *i;
 
-  for (re = recur_elist; re != 0; re = re->next)
-    recur_event_write (re, f);
+  LLIST_FOREACH (&recur_elist, i)
+    {
+      struct recur_event *rev = LLIST_GET_DATA (i);
+      recur_event_write (rev, f);
+    }
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (ra = recur_alist_p->root; ra != 0; ra = ra->next)
-    recur_apoint_write (ra, f);
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+  LLIST_TS_LOCK (&recur_alist_p);
+  LLIST_TS_FOREACH (&recur_alist_p, i)
+    {
+      struct recur_apoint *rapt = LLIST_GET_DATA (i);
+      recur_apoint_write (rapt, f);
+    }
+  LLIST_TS_UNLOCK (&recur_alist_p);
 }
 
 
@@ -590,12 +557,12 @@ recur_save_data (FILE *f)
  * diff_months and diff_years functions were provided by Lukas Fleischer to
  * correct the wrong calculation of recurrent dates after a turn of year.
  */
-#define bc(start, end, bs)                                              \
+#define BC(start, end, bs)                                              \
   (((end) - (start) + ((start) % bs) - ((end) % bs)) / bs               \
    + ((((start) % bs) == 0) ? 1 : 0))
 
-#define leapcount(start, end)                                           \
-  (bc(start, end, 4) - bc(start, end, 100) + bc(start, end, 400))
+#define LEAPCOUNT(start, end)                                           \
+  (BC(start, end, 4) - BC(start, end, 100) + BC(start, end, 400))
 
 
 /* Calculate the difference in days between two dates. */
@@ -612,7 +579,7 @@ diff_days (struct tm lt_start, struct tm lt_end)
   if (lt_end.tm_year > lt_start.tm_year)
     {
       diff += (lt_end.tm_year - lt_start.tm_year) * YEARINDAYS;
-      diff += leapcount (lt_start.tm_year + TM_YEAR_BASE,
+      diff += LEAPCOUNT (lt_start.tm_year + TM_YEAR_BASE,
                          lt_end.tm_year + TM_YEAR_BASE - 1);
     }
 
@@ -648,7 +615,13 @@ diff_years (struct tm lt_start, struct tm lt_end)
   return lt_end.tm_year - lt_start.tm_year;
 }
 
-/* 
+static int
+exc_inday (struct excp *exc, long day_start)
+{
+  return (exc->st >= day_start && exc->st < day_start + DAYINSEC);
+}
+
+/*
  * Check if the recurrent item belongs to the selected day,
  * and if yes, return the real start time.
  *
@@ -658,22 +631,20 @@ diff_years (struct tm lt_start, struct tm lt_end)
  * calculation of recurrent dates after a turn of years.
  */
 unsigned
-recur_item_inday (long item_start, struct days *item_exc, int rpt_type,
-		  int rpt_freq, long rpt_until, long day_start)
+recur_item_inday (long item_start, llist_t *item_exc, int rpt_type,
+                  int rpt_freq, long rpt_until, long day_start)
 {
   struct date start_date;
   long day_end, diff;
   struct tm lt_item, lt_day;
-  struct days *exc;
   time_t t;
 
   day_end = day_start + DAYINSEC;
   t = day_start;
   lt_day = *localtime (&t);
-  
-  for (exc = item_exc; exc != 0; exc = exc->next)
-    if (exc->st < day_end && exc->st >= day_start)
-      return (0);
+
+  if (LLIST_FIND_FIRST (item_exc, day_start, exc_inday))
+    return 0;
 
   if (rpt_until == 0)		/* we have an endless recurrent item */
     rpt_until = day_end;
@@ -683,26 +654,26 @@ recur_item_inday (long item_start, struct days *item_exc, int rpt_type,
 
   t = item_start;
   lt_item = *localtime (&t);
-  
+
   switch (rpt_type)
     {
     case RECUR_DAILY:
       diff = diff_days (lt_item, lt_day);
       if (diff % rpt_freq != 0)
-	return (0);
+        return (0);
       lt_item.tm_mday = lt_day.tm_mday;
       lt_item.tm_mon = lt_day.tm_mon;
       lt_item.tm_year = lt_day.tm_year;
       break;
     case RECUR_WEEKLY:
       if (lt_item.tm_wday != lt_day.tm_wday)
-	return (0);
+        return (0);
       else
-	{
+        {
           diff = diff_weeks (lt_item, lt_day);
-	  if (diff % rpt_freq != 0)
-	    return (0);
-	}
+          if (diff % rpt_freq != 0)
+            return (0);
+        }
       lt_item.tm_mday = lt_day.tm_mday;
       lt_item.tm_mon = lt_day.tm_mon;
       lt_item.tm_year = lt_day.tm_year;
@@ -710,14 +681,14 @@ recur_item_inday (long item_start, struct days *item_exc, int rpt_type,
     case RECUR_MONTHLY:
       diff = diff_months (lt_item, lt_day);
       if (diff % rpt_freq != 0)
-	return (0);
+        return (0);
       lt_item.tm_mon = lt_day.tm_mon;
       lt_item.tm_year = lt_day.tm_year;
       break;
     case RECUR_YEARLY:
       diff = diff_years (lt_item, lt_day);
       if (diff % rpt_freq != 0)
-	return (0);
+        return (0);
       lt_item.tm_year = lt_day.tm_year;
       break;
     default:
@@ -734,144 +705,123 @@ recur_item_inday (long item_start, struct days *item_exc, int rpt_type,
     return (0);
 }
 
-/* 
+unsigned
+recur_apoint_inday(struct recur_apoint *rapt, long day_start)
+{
+  return recur_item_inday (rapt->start, &rapt->exc, rapt->rpt->type,
+                           rapt->rpt->freq, rapt->rpt->until, day_start);
+}
+
+unsigned
+recur_event_inday(struct recur_event *rev, long day_start)
+{
+  return recur_item_inday (rev->day, &rev->exc, rev->rpt->type, rev->rpt->freq,
+                           rev->rpt->until, day_start);
+}
+
+/*
  * Delete a recurrent event from the list (if delete_whole is not null),
- * or delete only one occurence of the recurrent event. 
+ * or delete only one occurence of the recurrent event.
  */
 void
 recur_event_erase (long start, unsigned num, unsigned delete_whole,
-		   enum eraseflg flag)
+                   enum eraseflg flag)
 {
-  unsigned n = 0;
-  struct recur_event *i, **iptr;
+  llist_item_t *i;
 
-  iptr = &recur_elist;
-  for (i = recur_elist; i != 0; i = i->next)
+  i = LLIST_FIND_NTH (&recur_elist, num, start, recur_event_inday);
+
+  if (!i)
+    EXIT (_("event not found"));
+  struct recur_event *rev = LLIST_GET_DATA (i);
+
+  if (delete_whole)
     {
-      if (recur_item_inday (i->day, i->exc, i->rpt->type,
-			    i->rpt->freq, i->rpt->until, start))
-	{
-	  if (n == num)
-	    {
-	      if (delete_whole)
-		{
-                  switch (flag)
-                    {
-                    case ERASE_FORCE_ONLY_NOTE:
-                      erase_note (&i->note, flag);                      
-                      break;
-                    case ERASE_CUT:
-                      recur_event_free_bkp (ERASE_FORCE);
-                      recur_event_dup (i, &bkp_cut_recur_event);
-                      erase_note (&i->note, ERASE_FORCE_KEEP_NOTE);
-                      /* FALLTHROUGH */
-                    default:
-		      *iptr = i->next;
-		      mem_free (i->mesg);
-                      if (i->rpt)
-                        {
-                          mem_free (i->rpt);
-                          i->rpt = 0;
-                        }
-                      if (i->exc)
-                        {
-                          free_exc (i->exc);
-                          i->exc = 0;
-                        }
-                      if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
-                        erase_note (&i->note, flag);
-		      mem_free (i);
-                      break;
-		    }
-		  return;
-		}
-	      else
-		{
-                  recur_add_exc (&i->exc, start);
-		  return;
-		}
-	    }
-	  n++;
-	}
-      iptr = &i->next;
+      switch (flag)
+        {
+        case ERASE_FORCE_ONLY_NOTE:
+          erase_note (&rev->note, flag);
+          break;
+        case ERASE_CUT:
+          recur_event_free_bkp (ERASE_FORCE);
+          recur_event_dup (rev, &bkp_cut_recur_event);
+          erase_note (&rev->note, ERASE_FORCE_KEEP_NOTE);
+          /* FALLTHROUGH */
+        default:
+          LLIST_REMOVE (&recur_elist, i);
+          mem_free (rev->mesg);
+          if (rev->rpt)
+            {
+              mem_free (rev->rpt);
+              rev->rpt = 0;
+            }
+          free_exc_list (&rev->exc);
+          if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
+            erase_note (&rev->note, flag);
+          mem_free (rev);
+          break;
+        }
     }
-  EXIT (_("event not found"));
-  /* NOTREACHED */  
+  else
+    recur_add_exc (&rev->exc, start);
 }
 
 /*
  * Delete a recurrent appointment from the list (if delete_whole is not null),
- * or delete only one occurence of the recurrent appointment. 
+ * or delete only one occurence of the recurrent appointment.
  */
 void
 recur_apoint_erase (long start, unsigned num, unsigned delete_whole,
-		    enum eraseflg flag)
+                    enum eraseflg flag)
 {
-  unsigned n = 0;
-  struct recur_apoint *i, **iptr;
+  llist_item_t *i;
   int need_check_notify = 0;
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  iptr = &recur_alist_p->root;
-  for (i = recur_alist_p->root; i != 0; i = i->next)
+  i = LLIST_TS_FIND_NTH (&recur_alist_p, num, start, recur_apoint_inday);
+
+  if (!i)
+    EXIT (_("appointment not found"));
+  struct recur_apoint *rapt = LLIST_GET_DATA (i);
+
+  LLIST_TS_LOCK (&recur_alist_p);
+  if (notify_bar () && flag != ERASE_FORCE_ONLY_NOTE)
+    need_check_notify = notify_same_recur_item (rapt);
+  if (delete_whole)
     {
-      if (recur_item_inday (i->start, i->exc, i->rpt->type,
-			    i->rpt->freq, i->rpt->until, start))
-	{
-	  if (n == num)
-	    {
-	      if (notify_bar () && flag != ERASE_FORCE_ONLY_NOTE)
-		need_check_notify = notify_same_recur_item (i);
-	      if (delete_whole)
-		{
-                  switch (flag)
-                    {
-                    case ERASE_FORCE_ONLY_NOTE:
-                      erase_note (&i->note, flag);
-                      break;
-                    case ERASE_CUT:
-                      recur_apoint_free_bkp (ERASE_FORCE);
-                      recur_apoint_dup (i, &bkp_cut_recur_apoint);
-                      erase_note (&i->note, ERASE_FORCE_KEEP_NOTE);
-                      /* FALLTHROUGH */
-                    default:
-		      *iptr = i->next;
-		      mem_free (i->mesg);
-                      if (i->rpt)
-                        {
-                          mem_free (i->rpt);
-                          i->rpt = 0;
-                        }
-                      if (i->exc)
-                        {
-                          free_exc (i->exc);
-                          i->exc = 0;
-                        }
-                      if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
-                        erase_note (&i->note, flag);
-		      mem_free (i);
-		      pthread_mutex_unlock (&(recur_alist_p->mutex));
-		      if (need_check_notify)
-			notify_check_next_app ();
-                      break;
-		    }
-		  return;
-		}
-	      else
-		{
-                  recur_add_exc (&i->exc, start);
-		  pthread_mutex_unlock (&(recur_alist_p->mutex));
-		  if (need_check_notify)
-		    notify_check_next_app ();
-		  return;
-		}
-	    }
-	  n++;
-	}
-      iptr = &i->next;
+      switch (flag)
+        {
+        case ERASE_FORCE_ONLY_NOTE:
+          erase_note (&rapt->note, flag);
+          break;
+        case ERASE_CUT:
+          recur_apoint_free_bkp (ERASE_FORCE);
+          recur_apoint_dup (rapt, &bkp_cut_recur_apoint);
+          erase_note (&rapt->note, ERASE_FORCE_KEEP_NOTE);
+          /* FALLTHROUGH */
+        default:
+          LLIST_TS_REMOVE (&recur_alist_p, i);
+          mem_free (rapt->mesg);
+          if (rapt->rpt)
+            {
+              mem_free (rapt->rpt);
+              rapt->rpt = 0;
+            }
+          free_exc_list (&rapt->exc);
+          if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
+            erase_note (&rapt->note, flag);
+          mem_free (rapt);
+          if (need_check_notify)
+            notify_check_next_app (0);
+          break;
+        }
     }
-  EXIT (_("appointment not found"));
-  /* NOTREACHED */
+  else
+    {
+      recur_add_exc (&rapt->exc, start);
+      if (need_check_notify)
+        notify_check_next_app (0);
+    }
+  LLIST_TS_UNLOCK (&recur_alist_p);
 }
 
 /*
@@ -910,7 +860,6 @@ recur_repeat_item (struct conf *conf)
   int item_nb;
   struct day_item *p;
   struct recur_apoint *ra;
-  struct recur_event *re;
   long until, date;
 
   item_nb = apoint_hilt ();
@@ -918,12 +867,12 @@ recur_repeat_item (struct conf *conf)
   if (p->type != APPT && p->type != EVNT)
     {
       status_mesg (wrong_type_1, wrong_type_2);
-      ch = wgetch (win[STA].p);
+      (void)wgetch (win[STA].p);
       return;
     }
 
   while ((ch != 'D') && (ch != 'W') && (ch != 'M')
-	 && (ch != 'Y') && (ch != ESCAPE))
+         && (ch != 'Y') && (ch != ESCAPE))
     {
       status_mesg (mesg_type_1, mesg_type_2);
       ch = wgetch (win[STA].p);
@@ -936,24 +885,23 @@ recur_repeat_item (struct conf *conf)
   else
     {
       type = recur_char2def (ch);
-      ch = 0;
     }
 
   while (freq == 0)
     {
       status_mesg (mesg_freq_1, "");
       if (getstring (win[STA].p, user_input, BUFSIZ, 0, 1) == GETSTRING_VALID)
-	{
-	  freq = atoi (user_input);
-	  if (freq == 0)
-	    {
-	      status_mesg (mesg_wrong_freq, wrong_type_2);
-	      (void)wgetch (win[STA].p);
-	    }
-	  user_input[0] = '\0';
-	}
+        {
+          freq = atoi (user_input);
+          if (freq == 0)
+            {
+              status_mesg (mesg_wrong_freq, wrong_type_2);
+              (void)wgetch (win[STA].p);
+            }
+          user_input[0] = '\0';
+        }
       else
-	return;
+        return;
     }
 
   while (!date_entered)
@@ -962,60 +910,60 @@ recur_repeat_item (struct conf *conf)
                       DATEFMT_DESC (conf->input_datefmt));
       status_mesg (_(outstr), "");
       if (getstring (win[STA].p, user_input, BUFSIZ, 0, 1) == GETSTRING_VALID)
-	{
-	  if (strlen (user_input) == 1 && strncmp (user_input, "0", 1) == 0)
-	    {
-	      until = 0;
-	      date_entered = 1;
-	    }
-	  else
-	    {
-	      if (parse_date (user_input, conf->input_datefmt,
-			      &year, &month, &day))
-		{
-		  t = p->start;
-		  lt = localtime (&t);
-		  until_date.dd = day;
-		  until_date.mm = month;
-		  until_date.yyyy = year;
-		  until = date2sec (until_date, lt->tm_hour, lt->tm_min);
-		  if (until < p->start)
-		    {
-		      status_mesg (mesg_older, wrong_type_2);
-		      (void)wgetch (win[STA].p);
-		      date_entered = 0;
-		    }
-		  else
-		    {
-		      date_entered = 1;
-		    }
-		}
-	      else
-		{
-		  (void)snprintf (outstr, BUFSIZ, mesg_wrong_2,
+        {
+          if (strlen (user_input) == 1 && strncmp (user_input, "0", 1) == 0)
+            {
+              until = 0;
+              date_entered = 1;
+            }
+          else
+            {
+              if (parse_date (user_input, conf->input_datefmt,
+                              &year, &month, &day, calendar_get_slctd_day ()))
+                {
+                  t = p->start;
+                  lt = localtime (&t);
+                  until_date.dd = day;
+                  until_date.mm = month;
+                  until_date.yyyy = year;
+                  until = date2sec (until_date, lt->tm_hour, lt->tm_min);
+                  if (until < p->start)
+                    {
+                      status_mesg (mesg_older, wrong_type_2);
+                      (void)wgetch (win[STA].p);
+                      date_entered = 0;
+                    }
+                  else
+                    {
+                      date_entered = 1;
+                    }
+                }
+              else
+                {
+                  (void)snprintf (outstr, BUFSIZ, mesg_wrong_2,
                                   DATEFMT_DESC (conf->input_datefmt));
-		  status_mesg (mesg_wrong_1, _(outstr));
-		  (void)wgetch (win[STA].p);
-		  date_entered = 0;
-		}
-	    }
-	}
+                  status_mesg (mesg_wrong_1, _(outstr));
+                  (void)wgetch (win[STA].p);
+                  date_entered = 0;
+                }
+            }
+        }
       else
-	return;
+        return;
     }
 
   date = calendar_get_slctd_day_sec ();
   if (p->type == EVNT)
     {
-      re = recur_event_new (p->mesg, p->note, p->start, p->evnt_id,
-			    type, freq, until, NULL);
+      (void)recur_event_new (p->mesg, p->note, p->start, p->evnt_id,
+                             type, freq, until, NULL);
     }
   else if (p->type == APPT)
     {
       ra = recur_apoint_new (p->mesg, p->note, p->start, p->appt_dur,
-			     p->state, type, freq, until, NULL);
+                             p->state, type, freq, until, NULL);
       if (notify_bar ())
-	notify_check_repeated (ra);
+        notify_check_repeated (ra);
     }
   else
     {
@@ -1025,37 +973,40 @@ recur_repeat_item (struct conf *conf)
   day_erase_item (date, item_nb, ERASE_FORCE_KEEP_NOTE);
 }
 
-/* 
+/*
  * Read days for which recurrent items must not be repeated
  * (such days are called exceptions).
  */
-struct days *
-recur_exc_scan (FILE *data_file)
+void
+recur_exc_scan (llist_t *lexc, FILE *data_file)
 {
   int c = 0;
   struct tm day;
-  struct days *exc_head, *exc;
 
-  exc_head = 0;
+  LLIST_INIT (lexc);
   while ((c = getc (data_file)) == '!')
     {
       (void)ungetc (c, data_file);
       if (fscanf (data_file, "!%u / %u / %u ",
-		  &day.tm_mon, &day.tm_mday, &day.tm_year) != 3)
-	{
+                  &day.tm_mon, &day.tm_mday, &day.tm_year) != 3)
+        {
           EXIT (_("syntax error in item date"));
-	}
+        }
       day.tm_hour = 12;
       day.tm_min = day.tm_sec = 0;
       day.tm_isdst = -1;
       day.tm_year -= 1900;
       day.tm_mon--;
-      exc = mem_malloc (sizeof (struct days));
+      struct excp *exc = mem_malloc (sizeof (struct excp));
       exc->st = mktime (&day);
-      exc->next = exc_head;
-      exc_head = exc;
+      LLIST_ADD (lexc, exc);
     }
-  return exc_head;
+}
+
+static int
+recur_apoint_starts_before (struct recur_apoint *rapt, long time)
+{
+  return (rapt->start < time);
 }
 
 /*
@@ -1065,32 +1016,24 @@ recur_exc_scan (FILE *data_file)
 struct notify_app *
 recur_apoint_check_next (struct notify_app *app, long start, long day)
 {
-  struct recur_apoint *i;
+  llist_item_t *i;
   long real_recur_start_time;
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (i = recur_alist_p->root; i != 0; i = i->next)
+  LLIST_TS_LOCK (&recur_alist_p);
+  LLIST_TS_FIND_FOREACH (&recur_alist_p, app->time, recur_apoint_starts_before, i)
     {
-      if (i->start > app->time)
-	{
-	  pthread_mutex_unlock (&(recur_alist_p->mutex));
-	  return (app);
-	}
-      else
-	{
-	  real_recur_start_time =
-            recur_item_inday (i->start, i->exc, i->rpt->type, i->rpt->freq,
-                              i->rpt->until, day);
-	  if (real_recur_start_time > start)
-	    {
-	      app->time = real_recur_start_time;
-	      app->txt = mem_strdup (i->mesg);
-	      app->state = i->state;
-	      app->got_app = 1;
-	    }
-	}
+      struct recur_apoint *rapt = LLIST_TS_GET_DATA (i);
+
+      real_recur_start_time = recur_apoint_inday(rapt, day);
+      if (real_recur_start_time > start)
+        {
+          app->time = real_recur_start_time;
+          app->txt = mem_strdup (rapt->mesg);
+          app->state = rapt->state;
+          app->got_app = 1;
+        }
     }
-  pthread_mutex_unlock (&(recur_alist_p->mutex));
+  LLIST_TS_UNLOCK (&recur_alist_p);
 
   return (app);
 }
@@ -1099,25 +1042,13 @@ recur_apoint_check_next (struct notify_app *app, long start, long day)
 struct recur_apoint *
 recur_get_apoint (long date, int num)
 {
-  struct recur_apoint *o;
-  int n = 0;
+  llist_item_t *i = LLIST_TS_FIND_NTH (&recur_alist_p, num, date,
+                                      recur_apoint_inday);
+
+  if (i)
+    return LLIST_TS_GET_DATA (i);
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (o = recur_alist_p->root; o != 0; o = o->next)
-    {
-      if (recur_item_inday (o->start, o->exc, o->rpt->type,
-			    o->rpt->freq, o->rpt->until, date))
-	{
-	  if (n == num)
-	    {
-	      pthread_mutex_unlock (&(recur_alist_p->mutex));
-	      return (o);
-	    }
-	  n++;
-	}
-    }
   EXIT (_("item not found"));
-  return 0;
   /* NOTREACHED */
 }
 
@@ -1125,23 +1056,13 @@ recur_get_apoint (long date, int num)
 struct recur_event *
 recur_get_event (long date, int num)
 {
-  struct recur_event *o;
-  int n = 0;
+  llist_item_t *i = LLIST_FIND_NTH (&recur_elist, num, date,
+                                    recur_event_inday);
+
+  if (i)
+    return LLIST_GET_DATA (i);
 
-  for (o = recur_elist; o != 0; o = o->next)
-    {
-      if (recur_item_inday (o->day, o->exc, o->rpt->type,
-			    o->rpt->freq, o->rpt->until, date))
-	{
-	  if (n == num)
-	    {
-	      return (o);
-	    }
-	  n++;
-	}
-    }
   EXIT (_("item not found"));
-  return 0;
   /* NOTREACHED */
 }
 
@@ -1149,41 +1070,28 @@ recur_get_event (long date, int num)
 void
 recur_apoint_switch_notify (long date, int recur_nb)
 {
-  int n, need_chk_notify;
-  struct recur_apoint *o;
+  llist_item_t *i;
 
-  n = 0;
-  need_chk_notify = 0;
+  LLIST_TS_LOCK (&recur_alist_p);
+  i = LLIST_TS_FIND_NTH (&recur_alist_p, recur_nb, date, recur_apoint_inday);
 
-  pthread_mutex_lock (&(recur_alist_p->mutex));
-  for (o = recur_alist_p->root; o != 0; o = o->next)
-    {
-      if (recur_item_inday (o->start, o->exc, o->rpt->type,
-			    o->rpt->freq, o->rpt->until, date))
-	{
-	  if (n == recur_nb)
-	    {
-	      o->state ^= APOINT_NOTIFY;
+  if (!i)
+    EXIT (_("item not found"));
+  struct recur_apoint *rapt = LLIST_TS_GET_DATA (i);
 
-	      if (notify_bar ())
-		notify_check_repeated (o);
+  rapt->state ^= APOINT_NOTIFY;
 
-	      pthread_mutex_unlock (&(recur_alist_p->mutex));
-	      if (need_chk_notify)
-		notify_check_next_app ();
-	      return;
-	    }
-	  n++;
-	}
-    }
-  EXIT (_("item not found"));
-  /* NOTREACHED */
+  if (notify_bar ())
+    notify_check_repeated (rapt);
+
+  LLIST_TS_UNLOCK (&recur_alist_p);
 }
 
 void
 recur_event_paste_item (void)
 {
   long new_start, time_shift;
+  llist_item_t *i;
 
   new_start = date2sec (*calendar_get_slctd_day (), 12, 0);
   time_shift = new_start - bkp_cut_recur_event.day;
@@ -1191,14 +1099,12 @@ recur_event_paste_item (void)
   bkp_cut_recur_event.day += time_shift;
   if (bkp_cut_recur_event.rpt->until != 0)
     bkp_cut_recur_event.rpt->until += time_shift;
-  if (bkp_cut_recur_event.exc)
+  LLIST_FOREACH (&bkp_cut_recur_event.exc, i)
     {
-      struct days *exc;
-      
-      for (exc = bkp_cut_recur_event.exc; exc != 0; exc = exc->next)
-        exc->st += time_shift;
+      struct excp *exc = LLIST_GET_DATA (i);
+      exc->st += time_shift;
     }
-  
+
   (void)recur_event_new (bkp_cut_recur_event.mesg, bkp_cut_recur_event.note,
                          bkp_cut_recur_event.day, bkp_cut_recur_event.id,
                          bkp_cut_recur_event.rpt->type,
@@ -1212,6 +1118,7 @@ void
 recur_apoint_paste_item (void)
 {
   long new_start, time_shift;
+  llist_item_t *i;
 
   new_start = date2sec (*calendar_get_slctd_day (),
                         get_item_hour (bkp_cut_recur_apoint.start),
@@ -1221,14 +1128,12 @@ recur_apoint_paste_item (void)
   bkp_cut_recur_apoint.start += time_shift;
   if (bkp_cut_recur_apoint.rpt->until != 0)
     bkp_cut_recur_apoint.rpt->until += time_shift;
-  if (bkp_cut_recur_apoint.exc)
+  LLIST_FOREACH (&bkp_cut_recur_event.exc, i)
     {
-      struct days *exc;
-      
-      for (exc = bkp_cut_recur_apoint.exc; exc != 0; exc = exc->next)
-        exc->st += time_shift;
+      struct excp *exc = LLIST_GET_DATA (i);
+      exc->st += time_shift;
     }
-  
+
   (void)recur_apoint_new (bkp_cut_recur_apoint.mesg, bkp_cut_recur_apoint.note,
                           bkp_cut_recur_apoint.start, bkp_cut_recur_apoint.dur,
                           bkp_cut_recur_apoint.state,
@@ -1236,5 +1141,9 @@ recur_apoint_paste_item (void)
                           bkp_cut_recur_apoint.rpt->freq,
                           bkp_cut_recur_apoint.rpt->until,
                           &bkp_cut_recur_apoint.exc);
+
+  if (notify_bar ())
+    notify_check_repeated (&bkp_cut_recur_apoint);
+
   recur_apoint_free_bkp (ERASE_FORCE_KEEP_NOTE);
 }
