@@ -1,9 +1,7 @@
-/*	$calcurse: todo.c,v 1.36 2010-03-20 10:54:48 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -42,7 +40,7 @@
 
 #include "calcurse.h"
 
-struct todo *todolist;
+llist_t todolist;
 static int hilt = 0;
 static int todos = 0;
 static int first = 1;
@@ -52,15 +50,7 @@ static char *msgsav;
 static struct todo *
 todo_get_item (int item_number)
 {
-  struct todo *o;
-  int i;
-
-  o = todolist;
-  for (i = 1; i < item_number; i++)
-    {
-      o = o->next;
-    }
-  return (o);
+  return LLIST_GET_DATA (LLIST_NTH (&todolist, item_number - 1));
 }
 
 /* Sets which todo is highlighted. */
@@ -122,9 +112,9 @@ todo_first_decrease (void)
   first--;
 }
 
-/* 
+/*
  * Return the position of the hilghlighted item, relative to the first one
- * displayed. 
+ * displayed.
  */
 int
 todo_hilt_pos (void)
@@ -152,98 +142,76 @@ todo_new_item (void)
   if (getstring (win[STA].p, todo_input, BUFSIZ, 0, 1) == GETSTRING_VALID)
     {
       while ((ch < '1') || (ch > '9'))
-	{
-	  status_mesg (mesg_id, "");
-	  ch = wgetch (win[STA].p);
-	}
+        {
+          status_mesg (mesg_id, "");
+          ch = wgetch (win[STA].p);
+        }
       todo_add (todo_input, ch - '0', NULL);
       todos++;
     }
 }
 
+static int
+todo_cmp_id (struct todo *a, struct todo *b)
+{
+  /*
+   * As of version 2.6, todo items can have a negative id, which means they
+   * were completed. To keep them sorted, we need to consider the absolute id
+   * value.
+   */
+  int abs_a = abs (a->id);
+  int abs_b = abs (b->id);
+
+  return (abs_a < abs_b ? -1 : (abs_a == abs_b ? 0 : 1));
+}
+
 /*
  * Add an item in the todo linked list.
  */
 struct todo *
 todo_add (char *mesg, int id, char *note)
 {
-  struct todo *o, **i;
-  int absid;
-  
-  o = mem_malloc (sizeof (struct todo));
-  o->mesg = mem_strdup (mesg);
-  o->id = id;
-  o->note = (note != NULL && note[0] != '\0') ? mem_strdup (note) : NULL;
-  i = &todolist;
+  struct todo *todo;
 
-  /*
-   * As of version 2.6, todo items can have a negative id, which means they
-   * were completed. To keep them sorted, we need to consider the absolute id
-   * value.
-   */
-  absid = abs (id);
-  for (;;)
-    {
-      if (*i == 0 || abs ((*i)->id) > absid)
-	{
-	  o->next = *i;
-	  *i = o;
-	  break;
-	}
-      i = &(*i)->next;
-    }
-  return (o);
+  todo = mem_malloc (sizeof (struct todo));
+  todo->mesg = mem_strdup (mesg);
+  todo->id = id;
+  todo->note = (note != NULL && note[0] != '\0') ? mem_strdup (note) : NULL;
+
+  LLIST_ADD_SORTED (&todolist, todo, todo_cmp_id);
+
+  return todo;
 }
 
 /* Delete a note previously attached to a todo item. */
 static void
 todo_delete_note_bynum (unsigned num)
 {
-  unsigned n;
-  struct todo *i, **iptr;
+  llist_item_t *i = LLIST_NTH (&todolist, num);
 
-  n = 0;
-  iptr = &todolist;
-  for (i = todolist; i != 0; i = i->next)
-    {
-      if (n == num)
-	{
-	  if (i->note == NULL)
-            EXIT (_("no note attached"));
-	  erase_note (&i->note, ERASE_FORCE_ONLY_NOTE);
-	  return;
-	}
-      iptr = &i->next;
-      n++;
-    }
-  /* NOTREACHED */
-  EXIT (_("no such todo"));
+  if (!i)
+    EXIT (_("no such todo"));
+  struct todo *todo = LLIST_TS_GET_DATA (i);
+
+  if (!todo->note)
+    EXIT (_("no note attached"));
+  erase_note (&todo->note, ERASE_FORCE_ONLY_NOTE);
 }
 
 /* Delete an item from the todo linked list. */
 static void
 todo_delete_bynum (unsigned num, enum eraseflg flag)
 {
-  unsigned n;
-  struct todo *i, **iptr;
+  llist_item_t *i = LLIST_NTH (&todolist, num);
 
-  n = 0;
-  iptr = &todolist;
-  for (i = todolist; i != 0; i = i->next)
-    {
-      if (n == num)
-	{
-	  *iptr = i->next;
-	  mem_free (i->mesg);
-          erase_note (&i->note, flag);
-	  mem_free (i);
-	  return;
-	}
-      iptr = &i->next;
-      n++;
-    }
-  /* NOTREACHED */
-  EXIT (_("no such todo"));
+  if (!i)
+    EXIT (_("no such todo"));
+  struct todo *todo = LLIST_TS_GET_DATA (i);
+
+  LLIST_REMOVE (&todolist, i);
+  mem_free (todo->mesg);
+  erase_note (&todo->note, flag);
+  mem_free (todo);
 }
 
 /*
@@ -269,7 +237,7 @@ todo_delete (struct conf *conf)
   char *del_todo_str = _("Do you really want to delete this task ?");
   char *erase_warning =
       _("This item has a note attached to it. "
-	"Delete (t)odo or just its (n)ote ?");
+        "Delete (t)odo or just its (n)ote ?");
   char *erase_choice = _("[t/n] ");
   unsigned go_for_todo_del = 0;
   int answer, has_note;
@@ -279,14 +247,14 @@ todo_delete (struct conf *conf)
       status_mesg (del_todo_str, choices);
       answer = wgetch (win[STA].p);
       if ((answer == 'y') && (todos > 0))
-	{
-	  go_for_todo_del = 1;
-	}
+        {
+          go_for_todo_del = 1;
+        }
       else
-	{
-	  wins_erase_status_bar ();
-	  return;
-	}
+        {
+          wins_erase_status_bar ();
+          return;
+        }
     }
   else if (todos > 0)
     go_for_todo_del = 1;
@@ -314,11 +282,11 @@ todo_delete (struct conf *conf)
       todo_delete_bynum (hilt - 1, ERASE_FORCE);
       todos--;
       if (hilt > 1)
-	hilt--;
+        hilt--;
       if (todos == 0)
-	hilt = 0;
+        hilt = 0;
       if (hilt - first < 0)
-	first--;
+        first--;
       break;
     case 'n':
       todo_delete_note_bynum (hilt - 1);
@@ -329,34 +297,25 @@ todo_delete (struct conf *conf)
     }
 }
 
-/* 
+/*
  * Returns the position into the linked list corresponding to the
  * given todo item.
  */
 static int
-todo_get_position (struct todo *i)
+todo_get_position (struct todo *needle)
 {
-  struct todo *o;
-  int n = 1, found = 0;
+  llist_item_t *i;
+  int n = 0;
 
-  for (o = todolist; o; o = o->next)
+  LLIST_FOREACH (&todolist, i)
     {
-      if (o == i)
-	{
-	  found = 1;
-	  break;
-	}
       n++;
+      if (LLIST_TS_GET_DATA (i) == needle)
+        return n;
     }
-  if (found)
-    {
-      return (n);
-    }
-  else
-    {
-      EXIT (_("todo not found"));
-      return -1; /* avoid compiler warnings */
-    }
+
+  EXIT (_("todo not found"));
+  return -1; /* avoid compiler warnings */
 }
 
 /* Change an item priority by pressing '+' or '-' inside TODO panel. */
@@ -411,7 +370,7 @@ todo_edit_item (void)
 /* Display todo items in the corresponding panel. */
 static void
 display_todo_item (int incolor, char *msg, int prio, int note, int len, int y,
-		   int x)
+                   int x)
 {
   WINDOW *w;
   int ch_note;
@@ -423,7 +382,7 @@ display_todo_item (int incolor, char *msg, int prio, int note, int len, int y,
     snprintf (priostr, sizeof priostr, "%d", prio);
   else
     snprintf (priostr, sizeof priostr, "X");
-    
+
   if (incolor == 0)
     custom_apply_attr (w, ATTR_HIGHEST);
   if (strlen (msg) < len)
@@ -442,7 +401,7 @@ display_todo_item (int incolor, char *msg, int prio, int note, int len, int y,
 void
 todo_update_panel (int which_pan)
 {
-  struct todo *i;
+  llist_item_t *i;
   int len = win[TOD].w - 8;
   int num_todo = 0;
   int y_offset = 3, x_offset = 1;
@@ -455,20 +414,21 @@ todo_update_panel (int which_pan)
   /* Print todo item in the panel. */
   erase_window_part (win[TOD].p, 1, title_lines, win[TOD].w - 2,
                      win[TOD].h - 2);
-  for (i = todolist; i != 0; i = i->next)
+  LLIST_FOREACH (&todolist, i)
     {
+      struct todo *todo = LLIST_TS_GET_DATA (i);
       num_todo++;
       t_realpos = num_todo - first;
       incolor = num_todo - hilt;
       if (incolor == 0)
-	msgsav = i->mesg;
+        msgsav = todo->mesg;
       if (t_realpos >= 0 && t_realpos < max_items)
-	{
-	  display_todo_item (incolor, i->mesg, i->id,
-			     (i->note != NULL) ? 1 : 0, len, y_offset,
-			     x_offset);
-	  y_offset = y_offset + todo_lines;
-	}
+        {
+          display_todo_item (incolor, todo->mesg, todo->id,
+                             (todo->note != NULL) ? 1 : 0, len, y_offset,
+                             x_offset);
+          y_offset = y_offset + todo_lines;
+        }
     }
 
   /* Draw the scrollbar if necessary. */
@@ -481,9 +441,9 @@ todo_update_panel (int which_pan)
       int sbar_top = highend + title_lines;
 
       if ((sbar_top + sbar_length) > win[TOD].h - 1)
-	sbar_length = win[TOD].h - 1 - sbar_top;
+        sbar_length = win[TOD].h - 1 - sbar_top;
       draw_scrollbar (win[TOD].p, sbar_top, win[TOD].w - 2,
-		      sbar_length, title_lines, win[TOD].h - 1, hilt_bar);
+                      sbar_length, title_lines, win[TOD].h - 1, hilt_bar);
     }
 
   wnoutrefresh (win[TOD].p);
@@ -501,12 +461,15 @@ todo_edit_note (char *editor)
   if (i->note == NULL)
     {
       if ((filename = new_tempfile (path_notes, NOTESIZ)) != NULL)
-	i->note = filename;
+        i->note = filename;
       else
-	return;
+        return;
     }
   (void)snprintf (fullname, BUFSIZ, "%s%s", path_notes, i->note);
   wins_launch_external (fullname, editor);
+
+  if (io_file_is_empty (fullname) > 0)
+    erase_note (&i->note, ERASE_FORCE);
 }
 
 /* View a note previously attached to a todo */
@@ -523,20 +486,23 @@ todo_view_note (char *pager)
   wins_launch_external (fullname, pager);
 }
 
+void
+todo_free (struct todo *todo)
+{
+  mem_free (todo->mesg);
+  erase_note (&todo->note, ERASE_FORCE_KEEP_NOTE);
+  mem_free (todo);
+}
+
+void
+todo_init_list (void)
+{
+  LLIST_INIT (&todolist);
+}
+
 void
 todo_free_list (void)
 {
-  struct todo *o, **i;
-
-  i = &todolist;
-  while (*i)
-    {
-      o = *i;
-      *i = o->next;
-      mem_free (o->mesg);
-      erase_note (&o->note, ERASE_FORCE_KEEP_NOTE);
-      mem_free (o);
-    }
-  if (todolist)
-    mem_free (todolist);
+  LLIST_FREE_INNER (&todolist, todo_free);
+  LLIST_FREE (&todolist);
 }
