@@ -1,9 +1,7 @@
-/*	$calcurse: utils.c,v 1.84 2010-03-21 10:17:04 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -47,12 +45,14 @@
 
 #include "calcurse.h"
 
+#define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
+
 /* General routine to exit calcurse properly. */
 void
 exit_calcurse (int status)
 {
   int was_interactive;
-  
+
   if (ui_mode == UI_CURSES)
     {
       notify_stop_main_thread ();
@@ -64,7 +64,7 @@ exit_calcurse (int status)
     }
   else
     was_interactive = 0;
-  
+
   calendar_stop_date_thread ();
   io_stop_psave_thread ();
   free_user_data ();
@@ -78,7 +78,7 @@ exit_calcurse (int status)
       if (dmon.enable)
         dmon_start (status);
     }
-  
+
   exit (status);
 }
 
@@ -110,9 +110,9 @@ fatalbox (const char *errmsg)
   const int MSGLEN = WINCOL - 2;
   char msg[MSGLEN];
 
-  if (errmsg == 0)
+  if (errmsg == NULL)
     return;
-  
+
   (void)strncpy (msg, errmsg, MSGLEN);
   errwin = newwin (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
   custom_apply_attr (errwin, ATTR_HIGHEST);
@@ -132,14 +132,14 @@ warnbox (const char *msg)
 {
   WINDOW *warnwin;
   char *label = "/!\\";
-  const int WINROW = 10;  
+  const int WINROW = 10;
   const int WINCOL = col - 2;
   const int MSGLEN = WINCOL - 2;
   char displmsg[MSGLEN];
 
-  if (msg == 0)
+  if (msg == NULL)
     return;
-  
+
   (void)strncpy (displmsg, msg, MSGLEN);
   warnwin = newwin (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
   custom_apply_attr (warnwin, ATTR_HIGHEST);
@@ -153,7 +153,7 @@ warnbox (const char *msg)
   wins_doupdate ();
 }
 
-/* 
+/*
  * Print a message in the status bar.
  * Message texts for first line and second line are to be provided.
  */
@@ -170,17 +170,16 @@ status_mesg (char *mesg_line1, char *mesg_line2)
 /* Erase part of a window. */
 void
 erase_window_part (WINDOW *win, int first_col, int first_row, int last_col,
-		   int last_row)
+                   int last_row)
 {
   int c, r;
 
   for (r = first_row; r <= last_row; r++)
     {
       for (c = first_col; c <= last_col; c++)
-	{
-	  mvwprintw (win, r, c, " ");
-	}
+        mvwprintw (win, r, c, " ");
     }
+
   wnoutrefresh (win);
 }
 
@@ -207,7 +206,7 @@ popup (int pop_row, int pop_col, int pop_y, int pop_x, char *title, char *msg,
                any_key);
   custom_remove_attr (popup_win, ATTR_HIGHEST);
   wins_wrefresh (popup_win);
-  
+
   return popup_win;
 }
 
@@ -215,101 +214,75 @@ popup (int pop_row, int pop_col, int pop_y, int pop_x, char *title, char *msg,
 void
 print_in_middle (WINDOW *win, int starty, int startx, int width, char *string)
 {
-  int length, x, y;
-  float temp;
+  int len = strlen (string);
+  int x, y;
 
-  if (win == NULL)
-    win = stdscr;
+  win = win ? win : stdscr;
   getyx (win, y, x);
-  if (startx != 0)
-    x = startx;
-  if (starty != 0)
-    y = starty;
-  if (width == 0)
-    width = 80;
+  x = startx ? startx : x;
+  y = starty ? starty : y;
+  width = width ? width : 80;
+
+  x += (width - len) / 2;
 
-  length = strlen (string);
-  temp = (width - length) / 2;
-  x = startx + (int) temp;
   custom_apply_attr (win, ATTR_HIGHEST);
   mvwprintw (win, y, x, "%s", string);
   custom_remove_attr (win, ATTR_HIGHEST);
 }
 
-/* 
- * Draw the cursor at the correct position in string.
- * As echoing is not set, we need to know the string we are working on to 
- * handle display correctly.
- */
-static void
-showcursor (WINDOW *win, int y, int pos, char *str, int l, int offset)
-{
-  char *nc;
-
-  nc = str + pos;
-  wmove (win, y, pos - offset);
-  (pos >= l) ? waddch (win, SPACE | A_REVERSE) : waddch (win, *nc | A_REVERSE);
-}
-
 /* Print the string at the desired position. */
 static void
-showstring (WINDOW *win, int y, int x, char *str, int len, int pos)
+showstring (WINDOW *win, int x, int y, char *str, int len, int scroff,
+            int curpos)
 {
-  const int rec = 30, border = 3;
-  const int max_col = col - border, max_len = max_col - rec;
-  int page, max_page, offset, c = 0;
-  char *orig;
+  char c = 0;
 
-  orig = str;
-  max_page = (len - rec) / max_len;
-  page = (pos - rec) / max_len;
-  offset = page * max_len;
-  str += offset;
-  mvwaddnstr (win, y, x, str, MIN (len, max_col));
+  /* print string */
+  mvwaddnstr (win, y, x, &str[scroff], -1);
   wclrtoeol (win);
-  if (page > 0 && page < max_page)
-    {
-      c = '*';
-    }
-  else if (page > 0)
-    {
-      c = '<';
-    }
-  else if (page < max_page)
-    {
-      c = '>';
-    }
-  else
-    c = 0;
+
+  /* print scrolling indicator */
+  if (scroff > 0 && scroff < len - col)
+    c = '*';
+  else if (scroff > 0)
+    c = '<';
+  else if (scroff < len - col)
+    c = '>';
   mvwprintw (win, y, col - 1, "%c", c);
-  showcursor (win, y, pos, orig, len, offset);
+
+  /* print cursor */
+  wmove (win, y, curpos - scroff);
+
+  if (curpos >= len)
+    waddch (win, SPACE | A_REVERSE);
+  else
+    waddch (win, str[curpos] | A_REVERSE);
 }
 
 /* Delete a character at the given position in string. */
 static void
 del_char (int pos, char *str)
 {
-  int len;
-
   str += pos;
-  len = strlen (str) + 1;
-  memmove (str, str + 1, len);
+  memmove (str, str + 1, strlen (str) + 1);
 }
 
 /* Add a character at the given position in string. */
-static char *
-add_char (int pos, int ch, char *str)
+static void
+ins_char (int pos, int ch, char *str)
 {
-  int len;
-
   str += pos;
-  len = strlen (str) + 1;
-  memmove (str + 1, str, len);
+  memmove (str + 1, str, strlen (str) + 1);
   *str = ch;
-  return (str += len);
 }
 
-/* 
+static void
+bell (void)
+{
+  printf ("\a");
+}
+
+/*
  * Getstring allows to get user input and to print it on a window,
  * even if noecho() is on. This function is also used to modify an existing
  * text (the variable string can be non-NULL).
@@ -320,98 +293,96 @@ add_char (int pos, int ch, char *str)
 enum getstr
 getstring (WINDOW *win, char *str, int l, int x, int y)
 {
-  int ch, newpos, len = 0;
-  char *orig;
+  const int pgsize = col / 3;
+
+  int len = strlen (str);
+  int curpos = len;
+  int scroff = 0;
+  int ch;
 
-  orig = str;
   custom_apply_attr (win, ATTR_HIGHEST);
-  for (; *str; ++str, ++len)
-    ;
-  newpos = x + len;
-  showstring (win, y, x, orig, len, newpos);
 
-  while ((ch = wgetch (win)) != '\n')
-    {
-      switch (ch)
-	{
-	case KEY_BACKSPACE:	/* delete one character */
-	case 330:
-	case 127:
-	case CTRL ('H'):
-	  if (len > 0 && newpos > x)
-	    {
-	      --newpos;
-	      --len;
-	      if (newpos >= x + len)
-		--str;
-	      else		/* to be deleted inside string */
-		del_char (newpos, orig);
-	    }
-	  break;
+  for (;;) {
+    while (curpos < scroff)
+      scroff -= pgsize;
+    while (curpos >= scroff + col - 1)
+      scroff += pgsize;
 
-	case CTRL ('D'):	/* delete next character */
-	  if (newpos != (x + len))
-	    {
-	      --len;
-	      if (newpos >= x + len)
-		--str;
-	      else
-		del_char (newpos, orig);
-	    }
-	  else
-	    printf ("\a");
-	  break;
+    showstring (win, x, y, str, len, scroff, curpos);
+    wins_doupdate ();
 
-	case CTRL ('K'):	/* delete to end-of-line */
-	  str = orig + newpos;
-	  *str = 0;
-	  len -= (len - newpos);
-	  break;
+    if ((ch = wgetch (win)) == '\n') break;
+    switch (ch)
+      {
+        case KEY_BACKSPACE:     /* delete one character */
+        case 330:
+        case 127:
+        case CTRL ('H'):
+          if (curpos > 0)
+            {
+              del_char ((--curpos), str);
+              len--;
+            }
+          else
+            bell ();
+          break;
+        case CTRL ('D'):        /* delete next character */
+          if (curpos < len)
+            {
+              del_char (curpos, str);
+              len--;
+            }
+          else
+            bell ();
+          break;
+        case CTRL ('W'):        /* delete a word */
+          if (curpos > 0) {
+            while (curpos && str[curpos - 1] == ' ')
+              {
+                del_char ((--curpos), str);
+                len--;
+              }
+            while (curpos && str[curpos - 1] != ' ')
+              {
+                del_char ((--curpos), str);
+                len--;
+              }
+          }
+          else
+            bell ();
+          break;
+        case CTRL ('K'):        /* delete to end-of-line */
+          str[curpos] = 0;
+          len = curpos;
+          break;
+        case CTRL ('A'):        /* go to begginning of string */
+          curpos = 0;
+          break;
+        case CTRL ('E'):        /* go to end of string */
+          curpos = len;
+          break;
+        case KEY_LEFT:          /* move one char backward  */
+        case CTRL ('B'):
+          if (curpos > 0) curpos--;
+          break;
+        case KEY_RIGHT:         /* move one char forward */
+        case CTRL ('F'):
+          if (curpos < len) curpos++;
+          break;
+        case ESCAPE:            /* cancel editing */
+          return (GETSTRING_ESC);
+          break;
+        default:                /* insert one character */
+          if (len < l - 1)
+            {
+              ins_char ((curpos++), ch, str);
+              len++;
+            }
+      }
+  }
 
-	case CTRL ('A'):	/* go to begginning of string */
-	  newpos = x;
-	  break;
-
-	case CTRL ('E'):	/* go to end of string */
-	  newpos = x + len;
-	  break;
-
-	case KEY_LEFT:		/* move one char backward  */
-	case CTRL ('B'):
-	  if (newpos > x)
-	    newpos--;
-	  break;
-
-	case KEY_RIGHT:	/* move one char forward */
-	case CTRL ('F'):
-	  if (newpos < len)
-	    newpos++;
-	  break;
-
-	case ESCAPE:	/* cancel editing */
-	  return (GETSTRING_ESC);
-	  break;
-
-	default:		/* insert one character */
-	  if (len < l - 1)
-	    {
-	      if (newpos >= len)
-		{
-		  str = orig + newpos;
-		  *str++ = ch;
-		}
-	      else		// char is to be inserted inside string 
-		str = add_char (newpos, ch, orig);
-	      ++len;
-	      ++newpos;
-	    }
-
-	}
-      showstring (win, y, x, orig, len, newpos);
-      wins_doupdate ();
-    }
-  *str = 0;
   custom_remove_attr (win, ATTR_HIGHEST);
+
   return (len == 0 ? GETSTRING_RET : GETSTRING_VALID);
 }
 
@@ -419,114 +390,80 @@ getstring (WINDOW *win, char *str, int l, int x, int y)
 int
 updatestring (WINDOW *win, char **str, int x, int y)
 {
-  char *newstr;
-  int escape, len = strlen (*str) + 1;
+  int len = strlen (*str);
+  char *buf;
+  enum getstr ret;
 
-  EXIT_IF (len > BUFSIZ, _("Internal error: line too long"));
-  
-  newstr = mem_malloc (BUFSIZ);
-  (void) memcpy (newstr, *str, len);
-  escape = getstring (win, newstr, BUFSIZ, x, y);
-  if (!escape)
+  EXIT_IF (len + 1 > BUFSIZ, _("Internal error: line too long"));
+
+  buf = mem_malloc (BUFSIZ);
+  (void)memcpy (buf, *str, len + 1);
+
+  ret = getstring (win, buf, BUFSIZ, x, y);
+
+  if (ret == GETSTRING_VALID)
     {
-      len = strlen (newstr) + 1;
-      *str = mem_realloc (*str, len, 1);
-      EXIT_IF (*str == 0, _("out of memory"));
-      (void) memcpy (*str, newstr, len);
+      len = strlen (buf);
+      *str = mem_realloc (*str, len + 1, 1);
+      EXIT_IF (*str == NULL, _("out of memory"));
+      (void)memcpy (*str, buf, len + 1);
     }
-  mem_free (newstr);
-  return escape;
+
+  mem_free (buf);
+  return ret;
 }
 
 /* checks if a string is only made of digits */
 int
 is_all_digit (char *string)
 {
-  int digit, i;
-  int all_digit;
+  for (; *string; string++)
+    {
+      if (!isdigit ((int)*string))
+        return 0;
+    }
 
-  digit = 0;
-  all_digit = 0;
-
-  for (i = 0; i <= strlen (string); i++)
-    if (isdigit (string[i]) != 0)
-      digit++;
-  if (digit == strlen (string))
-    all_digit = 1;
-  return all_digit;
+  return 1;
 }
 
 /* Given an item date expressed in seconds, return its start time in seconds. */
 long
 get_item_time (long date)
 {
-  return (long)(get_item_hour (date) * HOURINSEC
-                + get_item_min (date) * MININSEC);
+  return (long)(get_item_hour (date) * HOURINSEC +
+    get_item_min (date) * MININSEC);
 }
 
 int
 get_item_hour (long date)
 {
-  struct tm *lt;
-  time_t t;
-
-  t = (time_t)date;
-  lt = localtime (&t);
-
-  return lt->tm_hour;
+  return (localtime ((time_t *)&date))->tm_hour;
 }
 
 int
 get_item_min (long date)
 {
-  struct tm *lt;
-  time_t t;
-
-  t = (time_t)date;
-  lt = localtime (&t);
-
-  return lt->tm_min;
+  return (localtime ((time_t *)&date))->tm_min;
 }
 
 long
 date2sec (struct date day, unsigned hour, unsigned min)
 {
-  struct tm start, *lt;
-  time_t tstart, t;
+  time_t t = now ();
+  struct tm start = *(localtime (&t));
 
-  t = time (NULL);
-  lt = localtime (&t);
-  start = *lt;
-
-  start.tm_mon = day.mm;
+  start.tm_mon = day.mm - 1;
   start.tm_mday = day.dd;
-  start.tm_year = day.yyyy;
+  start.tm_year = day.yyyy - 1900;
   start.tm_hour = hour;
   start.tm_min = min;
   start.tm_sec = 0;
   start.tm_isdst = -1;
-  start.tm_year -= 1900;
-  start.tm_mon--;
-  tstart = mktime (&start);
-  EXIT_IF (tstart == -1, _("failure in mktime"));
 
-  return tstart;
-}
+  t = mktime (&start);
+  EXIT_IF (t == -1, _("failure in mktime"));
 
-/* Return a string containing the hour of a given date in seconds. */
-char *
-date_sec2hour_str (long sec)
-{
-  const int TIME_LEN = 6;
-  struct tm *lt;
-  time_t t;
-  char *timestr;
-
-  t = sec;
-  lt = localtime (&t);
-  timestr = (char *) mem_malloc (TIME_LEN);
-  (void)snprintf (timestr, TIME_LEN, "%02u:%02u", lt->tm_hour, lt->tm_min);
-  return timestr;
+  return t;
 }
 
 /* Return a string containing the date, given a date in seconds. */
@@ -534,36 +471,28 @@ char *
 date_sec2date_str (long sec, char *datefmt)
 {
   struct tm *lt;
-  time_t t;
-  char *datestr;
-
-  datestr = (char *) mem_calloc (BUFSIZ, sizeof (char));
+  char *datestr = (char *) mem_calloc (BUFSIZ, sizeof (char));
 
   if (sec == 0)
     (void)snprintf (datestr, BUFSIZ, "0");
   else
     {
-      t = sec;
-      lt = localtime (&t);
+      lt = localtime ((time_t *)&sec);
       strftime (datestr, BUFSIZ, datefmt, lt);
     }
 
-  return (datestr);
+  return datestr;
 }
 
 /* Generic function to format date. */
 void
 date_sec2date_fmt (long sec, const char *fmt, char *datef)
 {
-  struct tm *lt;
-  time_t t;
-
-  t = sec;
-  lt = localtime (&t);
+  struct tm *lt = localtime ((time_t *)&sec);
   strftime (datef, BUFSIZ, fmt, lt);
 }
 
-/* 
+/*
  * Used to change date by adding a certain amount of days or weeks.
  */
 long
@@ -571,7 +500,7 @@ date_sec_change (long date, int delta_month, int delta_day)
 {
   struct tm *lt;
   time_t t;
-  
+
   t = date;
   lt = localtime (&t);
   lt->tm_mon += delta_month;
@@ -583,7 +512,7 @@ date_sec_change (long date, int delta_month, int delta_day)
   return t;
 }
 
-/* 
+/*
  * Return a long containing the date which is updated taking into account
  * the new time and date entered by the user.
  */
@@ -603,7 +532,7 @@ update_time_in_date (long date, unsigned hr, unsigned mn)
   return (new_date);
 }
 
-/* 
+/*
  * Returns the date in seconds from year 1900.
  * If no date is entered, current date is chosen.
  */
@@ -638,49 +567,35 @@ min2sec (unsigned minutes)
   return (minutes * MININSEC);
 }
 
-/* 
- * Checks if a time has a good format. 
+/*
+ * Checks if a time has a good format.
  * The format could be either HH:MM or H:MM or MM, and we should have:
  * 0 <= HH <= 24 and 0 <= MM < 999.
- * This function returns 1 if the entered time is correct and in 
+ * This function returns 1 if the entered time is correct and in
  * [h:mm] or [hh:mm] format, and 2 if the entered time is correct and entered
  * in [mm] format.
  */
 int
 check_time (char *string)
 {
-  int ok = 0;
-  char hour[] = "  ";
-  char minutes[] = "  ";
+  char *s = mem_strdup(string);
+  char *hour = strtok(s, ":");
+  char *min = strtok(NULL, ":");
+  int h, m;
+  int ret = 0;
 
-  if (((strlen (string) == 2) || (strlen (string) == 3))
-      && (isdigit (string[0]) != 0) && (isdigit (string[1]) != 0))
+  if (min)
     {
-      (void)strncpy (minutes, string, 2);
-      if (atoi (minutes) >= 0)
-	ok = 2;			/* [MM] format */
+      h = atoi (hour);
+      m = atoi (min);
+      if (h >= 0 && h < 24 && m >= 0 && m < MININSEC)
+        ret = 1;
     }
-  else if ((strlen (string) == 4) && (isdigit (string[0]) != 0)
-	   && (isdigit (string[2]) != 0) && (isdigit (string[3]) != 0)
-           && (string[1] == ':'))
-    {
-      (void)strncpy (hour, string, 1);
-      strncpy (minutes, string + 2, 2);
-      if ((atoi (hour) <= 24) && (atoi (hour) >= 0)
-	  && (atoi (minutes) < MININSEC) && (atoi (minutes) >= 0))
-	ok = 1;			/* [H:MM] format */
-    }
-  else if ((strlen (string) == 5) && (isdigit (string[0]) != 0)
-	   && (isdigit (string[1]) != 0) && (isdigit (string[3]) != 0)
-	   && (isdigit (string[4]) != 0) && (string[2] == ':'))
-    {
-      strncpy (hour, string, 2);
-      strncpy (minutes, string + 3, 2);
-      if ((atoi (hour) <= 24) && (atoi (hour) >= 0)
-	  && (atoi (minutes) < MININSEC) && (atoi (minutes) >= 0))
-	ok = 1;			/* [HH:MM] format */
-    }
-  return (ok);
+  else if (strlen(s) < 4 && is_all_digit(s) && atoi(s) > 0)
+    ret = 2;
+
+  mem_free(s);
+  return ret;
 }
 
 /*
@@ -689,7 +604,7 @@ check_time (char *string)
  */
 void
 draw_scrollbar (WINDOW *win, int y, int x, int length,
-		int bar_top, int bar_bottom, unsigned hilt)
+                int bar_top, int bar_bottom, unsigned hilt)
 {
   mvwvline (win, bar_top, x, ACS_VLINE, bar_bottom - bar_top);
   if (hilt)
@@ -702,13 +617,13 @@ draw_scrollbar (WINDOW *win, int y, int x, int length,
 }
 
 /*
- * Print an item (either an appointment, event, or todo) in a 
- * popup window. This is useful if an item description is too 
+ * Print an item (either an appointment, event, or todo) in a
+ * popup window. This is useful if an item description is too
  * long to fit in its corresponding panel window.
  */
 void
 item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
-	       char *pop_title)
+               char *pop_title)
 {
   WINDOW *popup_win, *pad;
   const int margin_left = 4, margin_top = 4;
@@ -720,7 +635,7 @@ item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
   if (strncmp (pop_title, _("Appointment"), 11) == 0)
     {
       mvwprintw (popup_win, margin_top, margin_left, "- %s -> %s",
-		 saved_a_start, saved_a_end);
+                 saved_a_start, saved_a_end);
     }
   mvwprintw (pad, 0, margin_left, "%s", msg);
   wmove (win[STA].p, 0, 0);
@@ -754,20 +669,16 @@ get_today (void)
 long
 now (void)
 {
-  time_t current_time;
-
-  current_time = time (NULL);
-  return (current_time);
+  return (long)time (NULL);
 }
 
 char *
 nowstr (void)
 {
-  static char buf[BUFSIZ];  
-  time_t now;
+  static char buf[BUFSIZ];
+  time_t t = now ();
 
-  (void)time (&now);
-  (void)strftime (buf, sizeof buf, "%a %b %d %T %Y", localtime (&now));
+  (void)strftime (buf, sizeof buf, "%a %b %d %T %Y", localtime (&t));
 
   return buf;
 }
@@ -815,9 +726,24 @@ print_bool_option_incolor (WINDOW *win, unsigned option, int pos_y, int pos_x)
   wins_doupdate ();
 }
 
-/* 
+
+/*
+ * Get the name of the default directory for temporary files.
+ */
+const char *
+get_tempdir (void)
+{
+  if (getenv ("TMPDIR"))
+    return getenv ("TMPDIR");
+  else if (P_tmpdir)
+    return P_tmpdir;
+  else
+    return "/tmp";
+}
+
+/*
  * Create a new unique file, and return a newly allocated string which contains
- * the random part of the file name. 
+ * the random part of the file name.
  */
 char *
 new_tempfile (const char *prefix, int trailing_len)
@@ -838,10 +764,10 @@ new_tempfile (const char *prefix, int trailing_len)
   if ((fd = mkstemp (fullname)) == -1 || (file = fdopen (fd, "w+")) == NULL)
     {
       if (fd != -1)
-	{
-	  unlink (fullname);
-	  close (fd);
-	}
+        {
+          unlink (fullname);
+          close (fd);
+        }
       ERROR_MSG (_("temporary file \"%s\" could not be created"), fullname);
       return (char *)0;
     }
@@ -871,84 +797,110 @@ erase_note (char **note, enum eraseflg flag)
 /*
  * Convert a string containing a date into three integers containing the year,
  * month and day.
+ *
+ * If a pointer to a date structure containing the current date is passed as
+ * last parameter ("slctd_date"), the function will accept several short forms,
+ * e.g. "26" for the 26th of the current month/year or "3/1" for Mar 01 (or Jan
+ * 03, depending on the date format) of the current year. If a null pointer is
+ * passed, short forms won't be accepted at all.
+ *
  * Returns 1 if sucessfully converted or 0 if the string is an invalid date.
  */
 int
 parse_date (char *date_string, enum datefmt datefmt, int *year, int *month,
-            int *day)
+            int *day, struct date *slctd_date)
 {
-  int in1, in2, in3;
-  int lyear, lmonth, lday;
+  char sep = (datefmt == DATEFMT_ISO) ? '-' : '/';
+  char *p;
+  int in[3] = {0, 0, 0}, n = 0;
+  int d, m, y;
 
-  if (date_string == 0)
+  if (!date_string)
     return 0;
 
-  if (datefmt == DATEFMT_ISO)
+  /* parse string into in[], read up to three integers */
+  for (p = date_string; *p; p++)
     {
-      if (sscanf (date_string, "%d - %d - %d", &in1, &in2, &in3) < 3)
-        return 0;
-    }
-  else
-    {
-      if (sscanf (date_string, "%d / %d / %d", &in1, &in2, &in3) < 3)
+      if (*p == sep)
+        {
+          if ((++n) > 2)
+            return 0;
+        }
+      else if ((*p >= '0') && (*p <= '9'))
+        in[n] = in[n] * 10 + (int)(*p - '0');
+      else
         return 0;
     }
+
+  if ((!slctd_date && n < 2) || in[n] == 0)
+    return 0;
+
+  /* convert into day, month and year, depending on the date format */
   switch (datefmt)
     {
-    case DATEFMT_MMDDYYYY:
-      lmonth = in1;
-      lday = in2;
-      lyear = in3;
-      break;
-    case DATEFMT_DDMMYYYY:
-      lday = in1;
-      lmonth = in2;
-      lyear = in3;
-      break;
-    case DATEFMT_YYYYMMDD:
-    case DATEFMT_ISO:      
-      lyear = in1;
-      lmonth = in2;
-      lday = in3;
-      break;
-    default:
-      return 0;
+      case DATEFMT_MMDDYYYY:
+        m = (n >= 1) ? in[0] : 0;
+        d = (n >= 1) ? in[1] : in[0];
+        y = in[2];
+        break;
+      case DATEFMT_DDMMYYYY:
+        d = in[0];
+        m = in[1];
+        y = in[2];
+        break;
+      case DATEFMT_YYYYMMDD:
+      case DATEFMT_ISO:
+        y = (n >= 2) ? in[n - 2] : 0;
+        m = (n >= 1) ? in[n - 1] : 0;
+        d = in[n];
+        break;
+      default:
+        return 0;
     }
-  
-  if (lyear < 1 || lyear > 9999
-      || lmonth < 1 || lmonth > 12
-      || lday < 1 || lday > 31)
+
+  if (slctd_date)
+    {
+      if (y > 0 && y < 100)
+        {
+          /* convert "YY" format into "YYYY" */
+          y += slctd_date->yyyy - slctd_date->yyyy % 100;
+        }
+      else if (n < 2)
+        {
+          /* set year and, optionally, month if short from is used */
+          y = slctd_date->yyyy;
+          if (n < 1) m = slctd_date->mm;
+        }
+    }
+
+  /* check if date is valid, take leap years into account */
+  if (y < 1902 || y > 2037 || m < 1 || m > 12 || d < 1 ||
+      d > days[m - 1] + (m == 2 && ISLEAP (y)) ? 1 : 0)
     return 0;
-  
-  if (year != NULL)
-    *year = lyear;
-  
-  if (month != NULL)
-    *month = lmonth;
-  
-  if (day != NULL)
-    *day = lday;
-  
+
+  if (year)
+    *year = y;
+  if (month)
+    *month = m;
+  if (day)
+    *day = d;
+
   return 1;
 }
 
-char *
+void
 str_toupper (char *s)
 {
-  int len;
-
-  for (len = 0; s && s[len]; len++)
-    s[len] = toupper (s[len]);
-  return s;
+  if (!s)
+    return;
+  for (; *s; s++)
+    *s = toupper (*s);
 }
 
 void
 file_close (FILE *f, const char *pos)
 {
-  int ret;
-  
-  ret = fclose (f);
-  EXIT_IF (ret != 0, _("Error when closing file at %s"), pos);
+  EXIT_IF ((fclose (f)) != 0, _("Error when closing file at %s"), pos);
 }
 
 /*
