@@ -1,9 +1,7 @@
-/*	$calcurse: vars.c,v 1.21 2010-03-20 10:54:49 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2008, 2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -45,6 +43,7 @@
  * variables to store window size
  */
 int col = 0, row = 0;
+int resize = 0;
 
 /* variable to tell if the terminal supports color */
 unsigned colorize = 0;
@@ -86,9 +85,9 @@ char *daynames[8] = {
   N_("Sun")
 };
 
-/* 
+/*
  * variables to store data path names, which are initialized in
- * io_init() 
+ * io_init()
  */
 char path_dir[] = "";
 char path_todo[] = "";
@@ -110,7 +109,7 @@ struct nbar nbar;
 struct dmon_conf dmon;
 
 /*
- * Variables init 
+ * Variables init
  */
 void
 vars_init (struct conf *conf)
