@@ -1,9 +1,7 @@
-/*	$calcurse: wins.c,v 1.33 2010-03-23 13:08:46 culot Exp $	*/
-
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2007-2010 Frederic Culot <frederic@culot.org>
+ * Copyright (c) 2004-2011 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,8 +29,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Send your feedback or comments to : calcurse@culot.org
- * Calcurse home page : http://culot.org/calcurse
+ * Send your feedback or comments to : misc@calcurse.org
+ * Calcurse home page : http://calcurse.org
  *
  */
 
@@ -84,7 +82,7 @@ int
 wins_refresh (void)
 {
   int rc;
-  
+
   if (!screen_acquire ())
     return ERR;
   rc = refresh ();
@@ -97,7 +95,7 @@ int
 wins_wrefresh (WINDOW *win)
 {
   int rc;
-  
+
   if (!win || !screen_acquire ())
     return ERR;
   rc = wrefresh (win);
@@ -150,7 +148,7 @@ wins_sbar_wperc (void)
   unsigned perc;
 
   perc = col ? (unsigned)(100 * sbarwidth / col + 1): 0;
-  
+
   return perc > SBARMAXWIDTHPERC ? SBARMAXWIDTHPERC : perc;
 }
 
@@ -227,7 +225,7 @@ static void
 wins_init_panels (void)
 {
   char label[BUFSIZ];
-  
+
   win[CAL].p = newwin (CALHEIGHT, wins_sbar_width (), win[CAL].y, win[CAL].x);
   (void)snprintf (label, BUFSIZ, _("Calendar"));
   wins_show (win[CAL].p, label);
@@ -242,7 +240,7 @@ wins_init_panels (void)
   (void)snprintf (label, BUFSIZ, _("ToDo"));
   wins_show (win[TOD].p, label);
 
-  /* Enable function keys (i.e. arrow keys) in those windows */  
+  /* Enable function keys (i.e. arrow keys) in those windows */
   keypad (win[CAL].p, TRUE);
   keypad (win[APP].p, TRUE);
   keypad (win[TOD].p, TRUE);
@@ -261,14 +259,14 @@ wins_init (void)
   ui_mode = UI_CURSES;
 }
 
-/* 
+/*
  * Create a new window and its associated pad, which is used to make the
  * scrolling faster.
  */
 void
 wins_scrollwin_init (struct scrollwin *sw)
 {
-  EXIT_IF (sw == 0, "null pointer");
+  EXIT_IF (sw == NULL, "null pointer");
   sw->win.p = newwin (sw->win.h, sw->win.w, sw->win.y, sw->win.x);
   sw->pad.p = newpad (sw->pad.h, sw->pad.w);
   sw->first_visible_line = 0;
@@ -279,7 +277,7 @@ wins_scrollwin_init (struct scrollwin *sw)
 void
 wins_scrollwin_delete (struct scrollwin *sw)
 {
-  EXIT_IF (sw == 0, "null pointer");
+  EXIT_IF (sw == NULL, "null pointer");
   delwin(sw->win.p);
   delwin(sw->pad.p);
 }
@@ -289,7 +287,7 @@ void
 wins_scrollwin_display (struct scrollwin *sw)
 {
   const int visible_lines = sw->win.h - sw->pad.y - 1;
-  
+
   if (sw->total_lines > visible_lines)
     {
       float ratio = ((float) visible_lines) / ((float) sw->total_lines);
@@ -335,7 +333,7 @@ wins_reinit_panels (void)
   wins_init_panels ();
 }
 
-/* 
+/*
  * Delete the existing windows and recreate them with their new
  * size and placement.
  */
@@ -357,10 +355,7 @@ wins_reinit (void)
 void
 wins_show (WINDOW *win, char *label)
 {
-  int startx, starty, height, width;
-
-  getbegyx (win, starty, startx);
-  getmaxyx (win, height, width);
+  int width = getmaxx (win);
 
   box (win, 0, 0);
   mvwaddch (win, 2, 0, ACS_LTEE);
@@ -370,7 +365,7 @@ wins_show (WINDOW *win, char *label)
   print_in_middle (win, 1, 0, width, label);
 }
 
-/* 
+/*
  * Get the screen size and recalculate the windows configurations.
  */
 void
@@ -402,7 +397,7 @@ wins_get_config (void)
 
   win[CAL].w = wins_sbar_width ();
   win[CAL].h = CALHEIGHT;
-  
+
   if (layout <= 4)
     {				/* APPOINTMENT is the biggest panel */
       win[APP].w = col - win[CAL].w;
@@ -577,7 +572,7 @@ wins_update_panels (void)
   calendar_update_panel (&win[CAL]);
 }
 
-/* 
+/*
  * Update all of the three windows and put a border around the
  * selected window.
  */
@@ -615,7 +610,7 @@ wins_launch_external (const char *file, const char *cmd)
   int len;
 
   /* Beware of space between cmd and file. */
-  len = strlen (file) + strlen (cmd) + 2;	
+  len = strlen (file) + strlen (cmd) + 2;
 
   p = (char *) mem_calloc (len, sizeof (char));
   if (snprintf (p, len, "%s %s", cmd, file) == -1)
@@ -645,12 +640,12 @@ wins_launch_external (const char *file, const char *cmd)
 #define NB_APP_CMDS	31	/* same thing while in appointment view */
 #define NB_TOD_CMDS	30	/* same thing while in todo view */
 #define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
-#define CMDS_PER_LINE	6	/* max number of commands per line */  
+#define CMDS_PER_LINE	6	/* max number of commands per line */
 
 static unsigned status_page;
 
-/* 
- * Draws the status bar. 
+/*
+ * Draws the status bar.
  * To add a keybinding, insert a new binding_t item, add it in the *binding
  * table, and update the NB_CAL_CMDS, NB_APP_CMDS or NB_TOD_CMDS defines,
  * depending on which panel the added keybind is assigned to.
@@ -668,13 +663,13 @@ wins_status_bar (void)
   struct binding quit   = {_("Quit"),     KEY_GENERIC_QUIT};
   struct binding save   = {_("Save"),     KEY_GENERIC_SAVE};
   struct binding cut    = {_("Cut"),      KEY_GENERIC_CUT};
-  struct binding paste  = {_("Paste"),    KEY_GENERIC_PASTE};    
+  struct binding paste  = {_("Paste"),    KEY_GENERIC_PASTE};
   struct binding chgvu  = {_("Chg Win"), KEY_GENERIC_CHANGE_VIEW};
-  struct binding import = {_("Import"),   KEY_GENERIC_IMPORT};  
+  struct binding import = {_("Import"),   KEY_GENERIC_IMPORT};
   struct binding export = {_("Export"),   KEY_GENERIC_EXPORT};
   struct binding togo   = {_("Go to"),    KEY_GENERIC_GOTO};
   struct binding othr   = {_("OtherCmd"), KEY_GENERIC_OTHER_CMD};
-  struct binding conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};  
+  struct binding conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};
   struct binding draw   = {_("Redraw"),   KEY_GENERIC_REDRAW};
   struct binding appt   = {_("Add Appt"), KEY_GENERIC_ADD_APPT};
   struct binding todo   = {_("Add Todo"), KEY_GENERIC_ADD_TODO};
@@ -686,22 +681,22 @@ wins_status_bar (void)
   struct binding nview  = {_("Nxt View"), KEY_GENERIC_SCROLL_DOWN};
   struct binding pview  = {_("Prv View"), KEY_GENERIC_SCROLL_UP};
   struct binding up     = {_("Up"),       KEY_MOVE_UP};
-  struct binding down   = {_("Down"),     KEY_MOVE_DOWN};  
+  struct binding down   = {_("Down"),     KEY_MOVE_DOWN};
   struct binding left   = {_("Left"),     KEY_MOVE_LEFT};
-  struct binding right  = {_("Right"),    KEY_MOVE_RIGHT};  
+  struct binding right  = {_("Right"),    KEY_MOVE_RIGHT};
   struct binding weekb  = {_("beg Week"), KEY_START_OF_WEEK};
   struct binding weeke  = {_("end Week"), KEY_END_OF_WEEK};
   struct binding add    = {_("Add Item"), KEY_ADD_ITEM};
   struct binding del    = {_("Del Item"), KEY_DEL_ITEM};
   struct binding edit   = {_("Edit Itm"), KEY_EDIT_ITEM};
-  struct binding view   = {_("View"),     KEY_VIEW_ITEM};  
+  struct binding view   = {_("View"),     KEY_VIEW_ITEM};
   struct binding flag   = {_("Flag Itm"), KEY_FLAG_ITEM};
   struct binding rept   = {_("Repeat"),   KEY_REPEAT_ITEM};
   struct binding enote  = {_("EditNote"), KEY_EDIT_NOTE};
   struct binding vnote  = {_("ViewNote"), KEY_VIEW_NOTE};
   struct binding rprio  = {_("Prio.+"),   KEY_RAISE_PRIORITY};
   struct binding lprio  = {_("Prio.-"),   KEY_LOWER_PRIORITY};
-  
+
   struct binding *binding[TOTAL_CMDS] = {
     /* calendar keys */
     &help, &quit, &save, &chgvu, &nview, &pview, &up, &down, &left, &right,
@@ -738,7 +733,6 @@ wins_other_status_page (int panel)
 {
   int nb_item, max_page;
 
-  nb_item = 0;
   switch (panel)
     {
     case CAL:
